#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################
__all__ = ["CurvesPropertiesTool"]

from taurus.external.qt import QtGui, Qt
from taurus.external.qt import QtCore
from taurus.qt.qtcore.configuration import BaseConfigurableClass
from taurus_pyqtgraph.curveproperties import (
    get_properties_from_curves,
    set_properties_on_curves,
    CurvesAppearanceChooser,
)
import pyqtgraph


class CurvesPropertiesTool(QtGui.QAction, BaseConfigurableClass):
    """
    This tool inserts an action in the menu of the :class:`pyqtgraph.PlotItem`
    to which it is attached to show a dialog for editing curve properties.
    It is implemented as an Action, and provides a method to attach it to a
    PlotItem.
    """

    def __init__(self, parent=None):
        BaseConfigurableClass.__init__(self)
        QtGui.QAction.__init__(self, "Plot configuration", parent)
        self.triggered.connect(self._onTriggered)
        self.plot_item = None
        self.Y2Axis = None
        self.registerConfigProperty(
            self._getCurveAppearanceProperties,
            self._setCurveAppearanceProperties,
            "CurveProperties",
        )
        self.registerConfigProperty(
            self._getBackgroundColor,
            self._setBackgroundColor,
            "PlotBackground",
        )

    def _getBackgroundColor(self):
        try:
            return self.plot_item.scene().parent().backgroundBrush().color()
        except Exception:
            import taurus

            taurus.debug("Cannot get plot background. Revert to 'default'")
            return "default"

    def _setBackgroundColor(self, color):
        self.plot_item.scene().parent().setBackground(color)

    def attachToPlotItem(self, plot_item, y2=None):
        """
        Use this method to add this tool to a plot

        :param plot_item: (PlotItem)
        :param y2: (Y2ViewBox) instance of the Y2Viewbox attached to plot_item
                   if the axis change controls are to be used
        """
        self.plot_item = plot_item
        menu = plot_item.getViewBox().menu
        menu.addAction(self)
        self.Y2Axis = y2

    def _onTriggered(self):
        props = self._getCurveAppearanceProperties()
        curves = self.getModifiableItems()

        dlg = Qt.QDialog(parent=self.parent())
        dlg.setWindowTitle("Plot Configuration")
        layout = Qt.QVBoxLayout()

        w = CurvesAppearanceChooser(
            parent=dlg,
            curvePropDict=props,
            curvesDict=curves,
            showButtons=True,
            plotItem=self.plot_item,
            Y2Axis=self.Y2Axis,
        )
        layout.addWidget(w)
        dlg.setLayout(layout)
        dlg.exec_()

    def getModifiableItems(self):
        data_items = self.plot_item.listDataItems()
        # checks in all ViewBoxes from plot_item,
        # looking for data_items (Curves).

        for item in self.plot_item.scene().items():
            if isinstance(item, pyqtgraph.ViewBox):
                for data in item.addedItems:
                    if data not in data_items:
                        data_items.append(data)

        # The dialog will ignore curves that define `._UImodifiable=False`
        modifiable_items = {}
        for item in data_items:
            if getattr(item, "_UImodifiable", True):
                modifiable_items[item.name()] = item
        return modifiable_items

    def _getCurveAppearanceProperties(self):
        return get_properties_from_curves(self.getModifiableItems())

    def _setCurveAppearanceProperties(self, props):
        curves = self.getModifiableItems()
        set_properties_on_curves(
            props, curves, plotItem=self.plot_item, y2Axis=self.Y2Axis
        )


if __name__ == "__main__":
    import sys
    import numpy
    import pyqtgraph as pg
    from taurus.qt.qtgui.tpg import TaurusPlotDataItem
    from taurus.qt.qtgui.application import TaurusApplication

    app = TaurusApplication()

    # a standard pyqtgraph plot_item
    w = pg.PlotWidget()

    # add legend to the plot, for that we have to give a name to plot items
    w.addLegend()

    # add a Y2 axis
    from taurus.qt.qtgui.tpg import Y2ViewBox

    y2ViewBox = Y2ViewBox()
    y2ViewBox.attachToPlotItem(w.getPlotItem())

    # adding a regular data item (non-taurus)
    c1 = pg.PlotDataItem(
        name="st plot",
        pen=dict(color="y", width=3, style=QtCore.Qt.DashLine),
        fillLevel=0.3,
        fillBrush="g",
    )

    c1.setData(numpy.arange(300) / 300.0)
    w.addItem(c1)

    # adding a taurus data item
    c2 = TaurusPlotDataItem(
        name="st2 plot", pen="r", symbol="o", symbolSize=10
    )
    c2.setModel("sys/tg_test/1/wave")

    w.addItem(c2)

    # attach tool to plot item of the PlotWidget
    tool = CurvesPropertiesTool()
    tool.attachToPlotItem(w.getPlotItem(), y2=y2ViewBox)

    w.show()

    # directly trigger the tool
    tool.trigger()

    sys.exit(app.exec_())



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""
A Qt dialog for choosing plot appearance (symbols and lines)
for a Pyqtgraph.PlotDataItem or taurus-derived class
like TaurusPlotDataItem

.. warning:: this is Work-in-progress. The API from this module may still
             change. Please
"""
from __future__ import absolute_import

# TODO: WIP

from builtins import str
from builtins import object

__all__ = [
    "CurveAppearanceProperties",
    "CurvesAppearanceChooser",
    "serialize_opts",
    "deserialize_opts",
    "get_properties_from_curves",
    "set_properties_on_curves",
    "set_y_axis_for_curve",
    "CURVE_COLORS",
]

import copy

from taurus import warning
from taurus.external.qt import Qt
from taurus.core.util.containers import CaselessDict
from taurus.qt.qtgui.util.ui import UILoadable
from .y2axis import Y2ViewBox, set_y_axis_for_curve
import pyqtgraph


class CONFLICT(object):
    """
    This is just a do-nothing class to be used to indicate that there are
    conflicting values when merging properties from different curves
    """

    pass


NamedLineStyles = {
    CONFLICT: "",
    Qt.Qt.NoPen: "No line",
    Qt.Qt.SolidLine: "_____",
    Qt.Qt.DashLine: "_ _ _",
    Qt.Qt.DotLine: ".....",
    Qt.Qt.DashDotLine: "_._._",
    Qt.Qt.DashDotDotLine: ".._..",
}

ReverseNamedLineStyles = {}
for k, v in NamedLineStyles.items():
    ReverseNamedLineStyles[v] = k

# TODO:allow the dialog to use this curve styles
NamedCurveStyles = {
    CONFLICT: "",
    None: "",
    "No curve": "No curve",
    "Lines": "Lines",
    "Sticks": "Sticks",
    "Steps": "Steps",
    "Dots": "Dots",
}

ReverseNamedCurveStyles = {}
for k, v in NamedCurveStyles.items():
    ReverseNamedCurveStyles[v] = k

NamedSymbolStyles = {
    CONFLICT: "",
    None: "No symbol",
    "o": "Circle",
    "s": "Square",
    "d": "Diamond",
    "t": "Down Triangle",
    "t1": "Up triangle",
    "t3": "Left Triangle",
    "t2": "Right Triangle",
    "+": "Cross",
    "star": "Star",
    "p": "Pentagon",
    "h": "Hexagon",
}

ReverseNamedSymbolStyles = {}
for k, v in NamedSymbolStyles.items():
    ReverseNamedSymbolStyles[v] = k

NamedColors = [
    "Red",
    "Blue",
    "Green",
    "Magenta",
    "Cyan",
    "Yellow",
    "Orange",
    "greenyellow",
    "Gray",
    "White",
    "Black",
]

kelly_colors_hex = [  # https://stackoverflow.com/a/4382138
    0xFFB300,  # Vivid Yellow
    0x803E75,  # Strong Purple
    0xFF6800,  # Vivid Orange
    0xA6BDD7,  # Very Light Blue
    0xC10020,  # Vivid Red
    0xCEA262,  # Grayish Yellow
    0x817066,  # Medium Gray
    # The following don't work well for people with defective color vision
    0x007D34,  # Vivid Green
    0xF6768E,  # Strong Purplish Pink
    0x00538A,  # Strong Blue
    # 0xFF7A5C, # Strong Yellowish Pink
    # 0x53377A, # Strong Violet
    # 0xFF8E00, # Vivid Orange Yellow
    0xB32851,  # Strong Purplish Red
    # 0xF4C800, # Vivid Greenish Yellow
    # 0x7F180D, # Strong Reddish Brown
    0x93AA00,  # Vivid Yellowish Green
    0x593315,  # Deep Yellowish Brown
    # 0xF13A13, # Vivid Reddish Orange
    # 0x232C16, # Dark Olive Green
]

CURVE_COLORS = [Qt.QColor(n) for n in NamedColors[:-2]]
# CURVE_COLORS = [Qt.QColor(n) for n in kelly_colors_hex]


@UILoadable
class CurvesAppearanceChooser(Qt.QWidget):
    """
    A widget for choosing plot appearance for one or more curves.
    The current curves properties are passed using the setCurvesProps()
    method using a dictionary with the following structure::

        curvePropDict={name1:prop1, name2:prop2,...}

    where propX is an instance of :class:`CurveAppearanceProperties`
    When applying, a signal is emitted and the chosen properties are made
    available in a similar dictionary. """

    NAME_ROLE = Qt.Qt.UserRole

    controlChanged = Qt.pyqtSignal()
    curveAppearanceChanged = Qt.pyqtSignal(object, list)
    CurveTitleEdited = Qt.pyqtSignal("QString", "QString")

    def __init__(
        self,
        parent=None,
        curvePropDict=None,
        showButtons=False,
        autoApply=False,
        curvesDict=None,
        plotItem=None,
        Y2Axis=None,
    ):
        super(CurvesAppearanceChooser, self).__init__(parent)
        self.loadUi()
        self.autoApply = autoApply
        self._curvesDict = curvesDict
        self.plotItem = plotItem
        self.Y2Axis = Y2Axis

        self.sStyleCB.insertItems(0, sorted(NamedSymbolStyles.values()))
        self.lStyleCB.insertItems(0, list(NamedLineStyles.values()))
        self.cStyleCB.insertItems(0, list(NamedCurveStyles.values()))
        self.sColorCB.addItem("")
        self.lColorCB.addItem("")
        self.cAreaDSB.setRange(float("-inf"), float("inf"))
        if not showButtons:
            self.applyBT.hide()
            self.resetBT.hide()
        for color in CURVE_COLORS:
            icon = self._colorIcon(color)
            self.sColorCB.addItem(icon, "", Qt.QColor(color))
            self.lColorCB.addItem(icon, "", Qt.QColor(color))
        self.__itemsDict = CaselessDict()
        self.setCurvesProps(curvePropDict)

        if self.plotItem is None:
            self.bckgndBT.setVisible(False)

        # connections.
        self.curvesLW.itemSelectionChanged.connect(
            self._onSelectedCurveChanged
        )
        self.curvesLW.itemChanged.connect(self._onItemChanged)
        self.applyBT.clicked.connect(self.onApply)
        self.resetBT.clicked.connect(self.onReset)
        self.sStyleCB.currentIndexChanged.connect(self._onSymbolStyleChanged)

        self.sStyleCB.currentIndexChanged.connect(self._onControlChanged)
        self.lStyleCB.currentIndexChanged.connect(self._onControlChanged)
        self.sColorCB.currentIndexChanged.connect(self._onControlChanged)
        self.lColorCB.currentIndexChanged.connect(self._onControlChanged)
        self.cStyleCB.currentIndexChanged.connect(self._onControlChanged)
        self.sSizeSB.valueChanged.connect(self._onControlChanged)
        self.lWidthSB.valueChanged.connect(self._onControlChanged)
        self.cAreaDSB.valueChanged.connect(self._onControlChanged)
        self.sFillCB.stateChanged.connect(self._onControlChanged)
        self.cFillCB.stateChanged.connect(self._onControlChanged)

        self.assignToY1BT.toggled[bool].connect(self.__onY1Toggled)
        self.assignToY2BT.toggled[bool].connect(self.__onY2Toggled)

        self.bckgndBT.clicked.connect(self.changeBackgroundColor)

        # Disabled button until future implementations
        self.changeTitlesBT.setEnabled(False)

        # disable the group box with the options for swap curves between Y axes
        if Y2Axis is None or plotItem is None:
            self.groupBox.setEnabled(False)

        self._onSelectedCurveChanged()
        self.axis = None

    def __onY1Toggled(self, checked):
        if checked:
            self.assignToY2BT.setChecked(False)

    def __onY2Toggled(self, checked):
        if checked:
            self.assignToY1BT.setChecked(False)

    def changeBackgroundColor(self):
        """Launches a dialog for choosing the plot widget background color
        """
        color = Qt.QColorDialog.getColor(
            initial=self.plotItem.scene().parent().backgroundBrush().color(),
            parent=self,
        )
        if Qt.QColor.isValid(color):
            self.plotItem.scene().parent().setBackground(color)

    def setCurvesProps(self, curvePropDict):
        """Populates the list of curves from the properties dictionary. It uses
        the curve title for display, and stores the curve name as the item data
        (with role=CurvesAppearanceChooser.NAME_ROLE)

        :param curvePropDict:  (dict) a dictionary whith keys=curvenames and
                               values= :class:`CurveAppearanceProperties`
                               object
        """
        self.curvePropDict = curvePropDict
        self._curvePropDictOrig = copy.deepcopy(curvePropDict)
        self.curvesLW.clear()
        self.__itemsDict = CaselessDict()
        for name, prop in self.curvePropDict.items():
            # create and insert the item
            item = Qt.QListWidgetItem(prop.title, self.curvesLW)
            self.__itemsDict[name] = item
            item.setData(self.NAME_ROLE, name)
            item.setToolTip("<b>Curve Name:</b> %s" % name)
            item.setFlags(
                Qt.Qt.ItemIsEnabled
                | Qt.Qt.ItemIsSelectable
                | Qt.Qt.ItemIsUserCheckable
                | Qt.Qt.ItemIsDragEnabled
                | Qt.Qt.ItemIsEditable
            )
        self.curvesLW.setCurrentRow(0)

    def _onItemChanged(self, item):
        """slot used when an item data has changed"""
        name = item.data(self.NAME_ROLE)
        previousTitle = self.curvePropDict[name].title
        currentTitle = item.text()
        if previousTitle != currentTitle:
            self.curvePropDict[name].title = currentTitle
            self.CurveTitleEdited.emit(name, currentTitle)

    def updateTitles(self, newTitlesDict=None):
        """
        Updates the titles of the curves that are displayed in the curves list.

        :param newTitlesDict: (dict<str,str>) dictionary with key=curve_name
                                and value=title
        """
        if newTitlesDict is None:
            return
        for name, title in newTitlesDict.items():
            self.curvePropDict[name].title = title
            self.__itemsDict[name].setText(title)

    def getSelectedCurveNames(self):
        """Returns the curve names for the curves selected at the curves list.

        *Note*: The names may differ from the displayed text, which
        corresponds to the curve titles (this method is what you likely need if
        you want to get keys to use in curves or curveProp dicts).

        :return: (string_list) the names of the selected curves
        """
        return [
            item.data(self.NAME_ROLE) for item in self.curvesLW.selectedItems()
        ]

    def showProperties(self, prop=None):
        """Updates the dialog to show the given properties.

        :param prop: (CurveAppearanceProperties) the properties object
                     containing what should be shown. If a given property is
                     set to CONFLICT, the corresponding plot_item will show a
                     "neutral" display
        """

        if prop is None:
            prop = self._shownProp
        # set the Style comboboxes
        self.sStyleCB.setCurrentIndex(
            self.sStyleCB.findText(NamedSymbolStyles[prop.sStyle])
        )
        self.lStyleCB.setCurrentIndex(
            self.lStyleCB.findText(NamedLineStyles[prop.lStyle])
        )
        self.cStyleCB.setCurrentIndex(
            self.cStyleCB.findText(NamedCurveStyles[prop.cStyle])
        )

        if prop.y2 is CONFLICT:
            self.assignToY1BT.setChecked(False)
            self.assignToY2BT.setChecked(False)
        elif prop.y2:
            self.assignToY2BT.setChecked(True)
        else:
            self.assignToY1BT.setChecked(True)

        # set sSize and lWidth spinboxes. if prop.sSize is None, it puts -1
        # (which is the special value for these switchhboxes)
        if prop.sSize is CONFLICT or prop.sStyle is None:
            self.sSizeSB.setValue(-1)
        else:
            self.sSizeSB.setValue(max(prop.sSize, -1))
        if prop.lWidth is CONFLICT:
            self.lWidthSB.setValue(-1)
        else:
            self.lWidthSB.setValue(max(prop.lWidth, -1))

        # Set the Color combo boxes. The item at index 0 is the empty one in
        # the comboboxes Manage unknown colors by including them
        if prop.sColor in (None, CONFLICT) or prop.sStyle is None:
            index = 0
        else:
            index = self.sColorCB.findData(Qt.QColor(prop.sColor))
        if index == -1:  # if the color is not supported, add it to combobox
            index = self.sColorCB.count()
            self.sColorCB.addItem(
                self._colorIcon(Qt.QColor(prop.sColor)),
                "",
                Qt.QColor(prop.sColor),
            )
        self.sColorCB.setCurrentIndex(index)
        if prop.lColor is None or prop.lColor is CONFLICT:
            index = 0
        else:
            index = self.lColorCB.findData(Qt.QColor(prop.lColor))
        if index == -1:  # if the color is not supported, add it to combobox
            index = self.lColorCB.count()
            self.lColorCB.addItem(
                self._colorIcon(Qt.QColor(prop.lColor)),
                "",
                Qt.QColor(prop.lColor),
            )
        self.lColorCB.setCurrentIndex(index)
        # set the Fill Checkbox. The prop.sFill value can be in 3 states: True,
        # False and None
        if prop.sFill is None or prop.sFill is CONFLICT:
            checkState = Qt.Qt.PartiallyChecked
        elif prop.sFill:
            checkState = Qt.Qt.Checked
        else:
            checkState = Qt.Qt.Unchecked
        self.sFillCB.setCheckState(checkState)
        # set the Area Fill Checkbox. The prop.cFill value can be in 3 states:
        # True, False and None
        if prop.cFill is CONFLICT:
            checkState = Qt.Qt.PartiallyChecked
            self.cAreaDSB.setValue(0.0)
        elif prop.cFill is None:
            checkState = Qt.Qt.Unchecked
            self.cAreaDSB.setValue(0.0)
        else:
            checkState = Qt.Qt.Checked
            self.cAreaDSB.setValue(prop.cFill)
        self.cFillCB.setCheckState(checkState)

    def _onControlChanged(self, *args):
        """
        Slot to be called whenever a control plot_item is changed. It emits a
        `controlChanged` signal and applies the change if in autoapply mode.
        It ignores any arguments passed
        """

        self.controlChanged.emit()
        if self.autoApply:
            self.onApply()

    def _onSelectedCurveChanged(self):
        """Updates the shown properties when the curve selection changes"""
        plist = [
            self.curvePropDict[name] for name in self.getSelectedCurveNames()
        ]
        if len(plist) == 0:
            plist = [CurveAppearanceProperties()]
            self.lineGB.setEnabled(False)
            self.symbolGB.setEnabled(False)
            self.otherGB.setEnabled(False)
        else:
            self.lineGB.setEnabled(True)
            self.symbolGB.setEnabled(True)
            self.otherGB.setEnabled(True)

        self._shownProp = CurveAppearanceProperties.merge(plist)
        self.showProperties(self._shownProp)

    def _onSymbolStyleChanged(self, text):
        """Slot called when the Symbol style is changed, to ensure that symbols
        are visible if you choose them

        :param text: (str) the new symbol style label
        """
        text = str(text)
        if self.sSizeSB.value() < 2 and text not in ["", "No symbol"]:
            # a symbol size of 0 is invisible and 1 means you should use
            # cStyle=dots
            self.sSizeSB.setValue(3)

    def getShownProperties(self):
        """Returns a copy of the currently shown properties and updates
        self._shownProp

        :return: (CurveAppearanceProperties)
        """
        prop = CurveAppearanceProperties()

        for name in self.getSelectedCurveNames():
            prop.title = self.curvePropDict[name].title

        # get the values from the Style comboboxes. Note that the empty string
        # ("") translates into CONFLICT
        prop.sStyle = ReverseNamedSymbolStyles[
            str(self.sStyleCB.currentText())
        ]
        prop.lStyle = ReverseNamedLineStyles[str(self.lStyleCB.currentText())]
        prop.cStyle = ReverseNamedCurveStyles[str(self.cStyleCB.currentText())]
        # get sSize and lWidth from the spinboxes (-1 means conflict)
        v = self.sSizeSB.value()
        if v == -1:
            prop.sSize = CONFLICT
        else:
            prop.sSize = v
        v = self.lWidthSB.value()
        if v == -1:
            prop.lWidth = CONFLICT
        else:
            prop.lWidth = v
        # Get the Color combo boxes. The item at index 0 is the empty one in
        # the comboboxes
        index = self.sColorCB.currentIndex()
        if index == 0:
            prop.sColor = CONFLICT
        else:
            prop.sColor = Qt.QColor(self.sColorCB.itemData(index))
        index = self.lColorCB.currentIndex()
        if index == 0:
            prop.lColor = CONFLICT
        else:
            prop.lColor = Qt.QColor(self.lColorCB.itemData(index))
        # get the sFill from the Checkbox.
        checkState = self.sFillCB.checkState()
        if checkState == Qt.Qt.PartiallyChecked:
            prop.sFill = CONFLICT
        else:
            prop.sFill = bool(checkState)
        # get the cFill from the Checkbox.
        checkState = self.cFillCB.checkState()
        if checkState == Qt.Qt.PartiallyChecked:
            prop.cFill = CONFLICT
        elif checkState == Qt.Qt.Checked:
            prop.cFill = self.cAreaDSB.value()
        else:
            prop.cFill = None

        # get the y2 state from the buttons
        y1 = self.assignToY1BT.isChecked()
        y2 = self.assignToY2BT.isChecked()
        if not y1 and not y2:
            prop.y2 = CONFLICT
        elif y1:
            prop.y2 = False
        elif y2:
            prop.y2 = True
        else:
            # both buttons should never be checked simultaneously
            raise RuntimeError("Inconsistent state of Y-axis buttons")

        # store the props
        self._shownProp = copy.deepcopy(prop)
        return copy.deepcopy(prop)

    def onApply(self):
        """Apply does 3 things:

            - It updates `self.curvePropDict` using the current values
              chosen in the dialog
            - It applies the properties to the curves (if the Chooser was
              initialized with the appropriate curvesDict)
            - It emits a curveAppearanceChanged signal that indicates the names
              of the curves that changed and the new properties.  (TODO)

        :return: (tuple<CurveAppearanceProperties,list>) a tuple containing the
                 curve properties and a list of the selected curve names (as a
                 list<str>)
        """
        names = self.getSelectedCurveNames()
        prop = self.getShownProperties()
        # Update self.curvePropDict for selected properties
        for n in names:
            self.curvePropDict[n] = CurveAppearanceProperties.merge(
                [self.curvePropDict[n], prop],
                conflict=CurveAppearanceProperties.inConflict_update_a,
            )
        # emit a (PyQt) signal telling what properties (first argument) need to
        # be applied to which curves (second argument)
        # self.curveAppearanceChanged.emit(prop, names)
        # return both values

        if self._curvesDict is not None:
            set_properties_on_curves(
                self.curvePropDict,
                self._curvesDict,
                plotItem=self.plotItem,
                y2Axis=self.Y2Axis,
            )
        return prop, names

    def onReset(self):
        """slot to be called when the reset action is triggered. It reverts to
        the original situation"""
        self.setCurvesProps(self._curvePropDictOrig)
        self.curvesLW.clearSelection()

    def _colorIcon(self, color, w=10, h=10):
        # to do: create a border
        pixmap = Qt.QPixmap(w, h)
        pixmap.fill(Qt.QColor(color))
        return Qt.QIcon(pixmap)


def get_properties_from_curves(curves):
    """Returns a dictionary of properties corresponding to the curves given
    in the `curves` dict.
    :param curves: dict whose values are :class:`PlotDataItem` instances
                   and whose keys arbitrarily identify a given curve
    :return: properties dict containing :class:`CurveAppearanceProperties`
             instances and whose keys match those in the `curves` dict
    """
    curves_prop = {}
    for key, item in curves.items():
        y2 = isinstance(item.getViewBox(), Y2ViewBox)

        opts = item.opts
        pen = pyqtgraph.mkPen(opts["pen"])
        # symbol_pen = pyqtgraph.mkPen(opts["symbolPen"])
        symbol_brush = pyqtgraph.mkBrush(opts["symbolBrush"])
        title = opts.get("name")
        sStyle = opts["symbol"]
        sSize = opts["symbolSize"]

        if sStyle is None:
            sColor = None
            sSize = -1
        else:
            sColor = symbol_brush.color()

        sFill = symbol_brush.color()
        if sFill is None or sStyle is None:
            sFill = False
        else:
            sFill = True

        lStyle = pen.style()
        lWidth = pen.width()
        lColor = pen.color()
        cStyle = None

        cFill = opts["fillLevel"]

        curve_appearance_properties = CurveAppearanceProperties(
            sStyle=sStyle,
            sSize=sSize,
            sColor=sColor,
            sFill=sFill,
            lStyle=lStyle,
            lWidth=lWidth,
            lColor=lColor,
            cStyle=cStyle,
            cFill=cFill,
            y2=y2,
            title=title,
        )
        curves_prop[key] = curve_appearance_properties
    return curves_prop


def set_properties_on_curves(properties, curves, plotItem=None, y2Axis=None):
    """
    Sets properties provided in the `properties` dict to curves provided in
    the `curves` dict. The association of a given curve with a property is
    done by matching the keys in the respective dictionaries.
    If both `plotItem` and `y2Axis` are passed, the curve will be moved to the
    ViewBox defined in the .y2 property
    :param properties: dict whose values are :class:`CurveAppearanceProperties`
                       instances and whose keys arbitrarily identify a
                       given curve
    :param curves: dict whose values are :class:`PlotDataItem` instances
                   and whose keys match those of properties (if a key in
                   `curves` does not exist in `properties`, it will bed)
    :param plotItem: The :class:`PlotItem` containing the dataItem.
    :param y2Axis: The :class:`Y2ViewBox` instance
 e skipp   """

    for key, dataItem in curves.items():
        try:
            prop = properties[key]
        except KeyError:
            warning("Cannot restore curve '%s' (no known properties)", key)
            continue
        sStyle = prop.sStyle
        sSize = prop.sSize
        sColor = prop.sColor
        sFill = prop.sFill
        lStyle = prop.lStyle
        lWidth = prop.lWidth
        lColor = prop.lColor
        cFill = prop.cFill
        y2 = prop.y2
        # title = properties.title

        dataItem.setPen(dict(style=lStyle, width=lWidth, color=lColor))
        if cFill is not None:
            dataItem.setFillLevel(cFill)
            try:
                cFillColor = Qt.QColor(lColor)
                cFillColor.setAlphaF(0.5)  # set to semitransparent
            except Exception:
                cFillColor = lColor
            dataItem.setFillBrush(cFillColor)
        else:
            dataItem.setFillLevel(None)

        dataItem.setSymbol(sStyle)
        # dataItem.setSymbolPen(pyqtgraph.mkPen(color=sColor))
        if sStyle is None or sSize < 0:
            dataItem.setSymbolSize(0)
        else:
            dataItem.setSymbolSize(sSize)

        if sFill:
            dataItem.setSymbolBrush(pyqtgraph.mkColor(sColor))
        else:
            dataItem.setSymbolBrush(None)

        # act on the ViewBoxes only if plotItem and y2Axis are given
        if plotItem and y2Axis:
            set_y_axis_for_curve(y2, dataItem, plotItem, y2Axis)


class CurveAppearanceProperties(object):
    """An object describing the appearance of a TaurusCurve"""

    propertyList = [
        "sStyle",
        "sSize",
        "sColor",
        "sFill",
        "lStyle",
        "lWidth",
        "lColor",
        "cStyle",
        "cFill",
        "y2",
        "title",
        "visible",
    ]

    def __init__(
        self,
        sStyle=CONFLICT,
        sSize=CONFLICT,
        sColor=CONFLICT,
        sFill=CONFLICT,
        lStyle=CONFLICT,
        lWidth=CONFLICT,
        lColor=CONFLICT,
        cStyle=CONFLICT,
        y2=CONFLICT,
        cFill=CONFLICT,
        title=CONFLICT,
        visible=CONFLICT,
    ):
        """
        Possible keyword arguments are:
            - sStyle= symbolstyle
            - sSize= int
            - sColor= color
            - sFill= bool
            - lStyle= linestyle
            - lWidth= int
            - lColor= color
            - cStyle= curvestyle
            - cFill= float or None
            - y2= bool
            - visible = bool
            - title= str

        Where:
            - color is a color that QColor() understands (i.e. a
              Qt.Qt.GlobalColor, a color name, or a Qt.Qcolor)
            - symbolstyle is a key of NamedSymbolStyles
            - linestyle is a key of Qt.Qt.PenStyle
            - curvestyle is a key of NamedCurveStyles
            - cFill can either be None (meaning not to fill) or a float that
              indicates the baseline from which to fill
            - y2 is True if the curve is associated to the y2 axis
        """
        self.sStyle = sStyle
        self.sSize = sSize
        self.sColor = sColor
        self.sFill = sFill
        self.lStyle = lStyle
        self.lWidth = lWidth
        self.lColor = lColor
        self.cStyle = cStyle
        self.cFill = cFill
        self.y2 = y2
        self.title = title
        self.visible = visible

    def __repr__(self):
        ret = "<CurveAppearanceProperties:"
        for p in self.propertyList:
            ret += " {}={}".format(p, getattr(self, p))
        ret += ">"
        return ret

    @staticmethod
    def inConflict_update_a(a, b):
        """
        This  function can be passed to CurvesAppearance.merge()
        if one wants to update prop1 with prop2 except for those
        attributes of prop2 that are set to CONFLICT
        """
        if b is CONFLICT:
            return a
        else:
            return b

    @staticmethod
    def inConflict_CONFLICT(a, b):
        """In case of conflict, returns CONFLICT"""
        return CONFLICT

    def conflictsWith(self, other, strict=True):
        """
        Compares itself with another CurveAppearanceProperties object
        and returns (a list of then names of) the attributes that are in
        conflict between the two
        """
        result = []
        for aname in self.propertyList:
            vself = getattr(self, aname)
            vother = getattr(other, aname)
            if vself != vother and (
                strict or not (CONFLICT in (vself, vother))
            ):
                result.append(aname)
        return result

    @classmethod
    def merge(cls, plist, attributes=None, conflict=None):
        """
        returns a CurveAppearanceProperties object formed by merging a list
        of other CurveAppearanceProperties objects

        :param plist: (sequence<CurveAppearanceProperties>) objects to be
        merged
        :param attributes: (sequence<str>) the name of the attributes to
                           consider for the merge. If None, all the attributes
                           will be merged
        :param conflict: (callable) a function that takes 2 objects (having a
                         different attribute) and returns a value that solves
                         the conflict. If None is given, any conflicting
                         attribute will be set to CONFLICT.

        :return: (CurveAppearanceProperties) merged properties
        """
        n = len(plist)
        if n < 1:
            raise ValueError("plist must contain at least 1 member")
        plist = copy.deepcopy(plist)
        if n == 1:
            return plist[0]
        if attributes is None:
            attributes = cls.propertyList
        if conflict is None:
            conflict = CurveAppearanceProperties.inConflict_CONFLICT
        p = CurveAppearanceProperties()
        for a in attributes:
            alist = [p.__getattribute__(a) for p in plist]
            p.__setattr__(a, alist[0])
            for ai in alist[1:]:
                if alist[0] != ai:
                    # print "MERGING:",a,alist[0],ai,conflict(alist[0],ai)
                    p.__setattr__(a, conflict(alist[0], ai))
                    break
        return p


def deserialize_opts(opts):
    """
    Deserialize opts dict to pass it to a PlotDataItem

    :param opts: (dict) serialized properties (as the output of
                 :meth:`deserialize_opts`)

    :return: (dict) deserialized properties (acceptable by PlotDataItem)
    """
    # pen property
    if opts["pen"] is not None:
        opts["pen"] = _unmarshallingQPainter(opts, "pen", "pen")

    # shadowPen property
    if opts["shadowPen"] is not None:
        opts["shadowPen"] = _unmarshallingQPainter(opts, "shadowPen", "pen")

    # symbolPen property
    if opts["symbolPen"] is not None:
        opts["symbolPen"] = _unmarshallingQPainter(opts, "symbolPen", "pen")

    # fillBrush property
    if opts["fillBrush"] is not None:
        opts["fillBrush"] = _unmarshallingQPainter(opts, "fillBrush", "brush")

    # symbolBrush property
    if opts["symbolBrush"] is not None:
        opts["symbolBrush"] = _unmarshallingQPainter(
            opts, "symbolBrush", "brush"
        )

    return opts


def serialize_opts(opts):
    """
    Serialize all properties from PlotDataItem.

    :param opts: (dict) PlotDataItem opts (may include non-serializable
                 objects)

    :return: (dict) serialized properties (can be pickled)
    """
    # pen property (QPen object)
    if opts["pen"] is not None:
        _marshallingQPainter(opts, "pen", "pen")

    # shadowPen property (QPen object)
    if opts["shadowPen"] is not None:
        _marshallingQPainter(opts, "shadowPen", "pen")

    # symbolPen property (QPen object)
    if opts["symbolPen"] is not None:
        _marshallingQPainter(opts, "symbolPen", "pen")

    # fillBrush property (QBrush object)
    if opts["fillBrush"] is not None:
        _marshallingQPainter(opts, "fillBrush", "brush")

    # symbolBrush property (QBrush object)
    if opts["symbolBrush"] is not None:
        _marshallingQPainter(opts, "symbolBrush", "brush")

    return opts


def _marshallingQPainter(opts, prop_name, qPainter):
    if qPainter == "pen":
        painter = pyqtgraph.mkPen(opts[prop_name])
        opts[prop_name + "_width"] = painter.width()
        opts[prop_name + "_dash"] = painter.dashPattern()
        opts[prop_name + "_cosmetic"] = painter.isCosmetic()
    elif qPainter == "brush":
        painter = pyqtgraph.mkBrush(opts[prop_name])
    else:
        return

    color = pyqtgraph.colorStr(painter.color())
    opts[prop_name] = color
    opts[prop_name + "_style"] = painter.style()


def _unmarshallingQPainter(opts, prop_name, qPainter):
    color = opts[prop_name]
    style = opts[prop_name + "_style"]
    del opts[prop_name + "_style"]

    if qPainter == "pen":
        width = opts[prop_name + "_width"]
        dash = opts[prop_name + "_dash"]
        cosmetic = opts[prop_name + "_cosmetic"]
        del opts[prop_name + "_width"]
        del opts[prop_name + "_dash"]
        del opts[prop_name + "_cosmetic"]
        painter = pyqtgraph.mkPen(
            color=color, style=style, width=width, dash=dash, cosmetic=cosmetic
        )
    elif qPainter == "brush":
        painter = pyqtgraph.mkBrush(color=color)
        painter.setStyle(style)
    else:
        return

    return painter



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""Example on using a tpg.TaurusPlotDataItem on a pure pyqtgraph plot"""

if __name__ == "__main__":
    import sys
    import numpy
    from taurus.qt.qtgui.application import TaurusApplication
    from taurus.qt.qtgui.tpg import TaurusPlotDataItem
    import pyqtgraph as pg

    app = TaurusApplication()

    # a standard pyqtgraph plot_item
    w = pg.PlotWidget()

    # add legend to the plot, for that we have to give a name to plot items
    w.addLegend()

    # add a regular data item (non-taurus)
    c1 = pg.PlotDataItem(name="pg item", pen="b", fillLevel=0, brush="c")
    c1.setData(numpy.linspace(0, 2, 250))
    w.addItem(c1)

    # add a taurus data item
    c2 = TaurusPlotDataItem(name="taurus item", pen="r", symbol="o")
    c2.setModel('eval:Quantity(rand(256),"m")')
    w.addItem(c2)

    w.show()

    sys.exit(app.exec_())



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################
from __future__ import absolute_import

__all__ = ["TaurusTrend"]

from future.utils import string_types
import copy

from taurus.external.qt import QtGui, Qt
from taurus.core.util.containers import LoopList
from taurus.qt.qtcore.configuration import BaseConfigurableClass
from taurus.core.util.log import Logger

from pyqtgraph import PlotWidget

from .curvespropertiestool import CurvesPropertiesTool
from .dateaxisitem import DateAxisItem
from .legendtool import PlotLegendTool
from .forcedreadtool import ForcedReadTool
from .datainspectortool import DataInspectorTool
from .taurusmodelchoosertool import TaurusXYModelChooserTool
from .taurustrendset import TaurusTrendSet
from .y2axis import Y2ViewBox
from .autopantool import XAutoPanTool
from .curveproperties import CURVE_COLORS


class TaurusTrend(PlotWidget, BaseConfigurableClass):
    """
    TaurusTrend is a general widget for plotting the evolution of a value
    over time. It is an extended taurus-aware version of
    :class:`pyqtgraph.PlotWidget`.

    Apart from all the features already available in a regulat PlotWidget,
    TaurusTrend incorporates the following tools/features:

        - Secondary Y axis (right axis)
        - Time X axis
        - A plot configuration dialog, and save/restore configuration
          facilities
        - A menu option for adding/removing taurus  models
        - A menu option for showing/hiding the legend
        - Automatic color change of curves for newly added models

    """

    def __init__(self, parent=None, **kwargs):

        if Qt.QT_VERSION < 0x050000:
            # Workaround for issue when using super with pyqt<5
            BaseConfigurableClass.__init__(self)
            PlotWidget.__init__(self, parent=parent, **kwargs)
        else:
            super(TaurusTrend, self).__init__(parent=parent, **kwargs)

        # Compose with a Logger
        self._logger = Logger(name=self.__class__.__name__)
        self.debug = self._logger.debug
        self.info = self._logger.info
        self.warning = self._logger.warning
        self.error = self._logger.error

        # set up cyclic color generator
        self._curveColors = LoopList(CURVE_COLORS)
        self._curveColors.setCurrentIndex(-1)

        plot_item = self.getPlotItem()
        menu = plot_item.getViewBox().menu

        # add trends clear action
        clearTrendsAction = QtGui.QAction("Clear trends", menu)
        clearTrendsAction.triggered.connect(self.clearTrends)
        menu.addAction(clearTrendsAction)

        # add save & retrieve configuration actions
        saveConfigAction = QtGui.QAction("Save configuration", menu)
        saveConfigAction.triggered.connect(self.saveConfigFile)
        menu.addAction(saveConfigAction)

        loadConfigAction = QtGui.QAction("Retrieve saved configuration", menu)
        loadConfigAction.triggered.connect(self.loadConfigFile)
        menu.addAction(loadConfigAction)

        self.registerConfigProperty(self._getState, self.restoreState, "state")

        # add legend tool
        legend_tool = PlotLegendTool(self)
        legend_tool.attachToPlotItem(plot_item)

        # add model chooser
        self._model_chooser_tool = TaurusXYModelChooserTool(
            self, itemClass=TaurusTrendSet, showX=False
        )
        self._model_chooser_tool.attachToPlotItem(
            self.getPlotItem(), self, self._curveColors
        )

        # add Y2 axis
        self._y2 = Y2ViewBox()
        self._y2.attachToPlotItem(plot_item)

        # Add time X axis
        axis = DateAxisItem(orientation="bottom")
        axis.attachToPlotItem(plot_item)

        # add plot configuration dialog
        self._cprop_tool = CurvesPropertiesTool(self)
        self._cprop_tool.attachToPlotItem(plot_item, y2=self._y2)

        # add data inspector widget
        inspector_tool = DataInspectorTool(self)
        inspector_tool.attachToPlotItem(self.getPlotItem())

        # add force read tool
        self._fr_tool = ForcedReadTool(self)
        self._fr_tool.attachToPlotItem(self.getPlotItem())

        # Add the auto-pan ("oscilloscope mode") tool
        self._autopan = XAutoPanTool()
        self._autopan.attachToPlotItem(self.getPlotItem())

        # Register config properties
        self.registerConfigDelegate(self._model_chooser_tool, "XYmodelchooser")
        # self.registerConfigDelegate(self._y2, "Y2Axis")
        self.registerConfigDelegate(self._cprop_tool, "CurvePropertiesTool")
        self.registerConfigDelegate(legend_tool, "legend")
        self.registerConfigDelegate(self._fr_tool, "forceread")
        self.registerConfigDelegate(inspector_tool, "inspector")

    # --------------------------------------------------------------------
    # workaround for bug in pyqtgraph v<=0.10.0, already fixed in
    # https://github.com/pyqtgraph/pyqtgraph/commit/52754d4859
    # TODO: remove this once pyqtgraph v>0.10 is released
    def __getattr__(self, item):
        try:
            return PlotWidget.__getattr__(self, item)
        except NameError:
            raise AttributeError(
                "{} has no attribute {}".format(self.__class__.__name__, item)
            )

    # --------------------------------------------------------------------

    def __getitem__(self, idx):
        """
        Provides a list-like interface: items can be accessed using slice
        notation
        """
        return self._getCurves()[idx]

    def __len__(self):
        return len(self._getCurves())

    def _getCurves(self):
        """returns a flat list with all items from all trend sets"""
        ret = []
        for ts in self.getTrendSets():
            ret += ts[:]
        return ret

    def getTrendSets(self):
        return [
            e
            for e in self.getPlotItem().listDataItems()
            if isinstance(e, TaurusTrendSet)
        ]

    def clearTrends(self):
        for ts in self.getTrendSets():
            ts.clearBuffer()

    def setModel(self, names):
        """Set a list of models"""
        # support passing a string  in names instead of a sequence
        if isinstance(names, string_types):
            names = [names]
        self._model_chooser_tool.updateModels(names or [])

    def addModels(self, names):
        """Reimplemented to delegate to the  model chooser"""
        # support passing a string in names
        if isinstance(names, string_types):
            names = [names]
        self._model_chooser_tool.addModels(names)

    def _getState(self):
        """Same as PlotWidget.saveState but removing viewRange conf to force
        a refresh with targetRange when loading
        """
        state = copy.deepcopy(self.saveState())
        # remove viewRange conf
        del state["view"]["viewRange"]
        return state

    def setXAxisMode(self, x_axis_mode):
        """Required generic TaurusTrend API """
        if x_axis_mode != "t":
            raise NotImplementedError(  # TODO
                'X mode "{}" not yet supported'.format(x_axis_mode)
            )

    def setForcedReadingPeriod(self, period):
        """Required generic TaurusTrend API """
        self._fr_tool.setPeriod(period)

    def setMaxDataBufferSize(self, buffer_size):
        """Required generic TaurusTrend API """
        raise NotImplementedError(  # TODO
            "Setting the max buffer size is not yet supported by tpg trend"
        )


def trend_main(
    models=(), config_file=None, demo=False, window_name="TaurusTrend (pg)"
):
    """Launch a TaurusTrend"""
    import sys
    from taurus.qt.qtgui.application import TaurusApplication

    app = TaurusApplication(cmd_line_parser=None, app_name="taurustrend(pg)")

    w = TaurusTrend()

    w.setWindowTitle(window_name)

    if demo:
        models = list(models)
        models.extend(["eval:rand()", "eval:1+rand(2)"])

    if config_file is not None:
        w.loadConfigFile(config_file)

    if models:
        w.setModel(models)

    w.show()
    ret = app.exec_()
    # w.saveConfigFile('tmp/TaurusTrend.pck')
    # import pprint
    # pprint.pprint(w.createConfig())

    sys.exit(ret)


if __name__ == "__main__":
    trend_main(models=("eval:rand()", "sys/tg_test/1/ampli"))



def unique_data_item_name(name, existing):
    """Accepts a name and a list of existing names and
    returns a name qhich may be modified with a numerical
    suffix such as " (1)", or " (2)", etc in order to avoid
    duplications with the existing names
    """
    base = name
    i = 1
    while name in existing:
        i += 1
        name = "{} ({})".format(base, i)
    return name


def ensure_unique_curve_name(dataItem, plotItem):
    """Given a plotItem, it changes its name adding a prefix as in
    `unique_data_item_name`" to avoid duplication of names with respect
    to the plotDataItems contained in Sardana
    """
    name = dataItem.name()
    if name is None:
        return dataItem
    existing = [e.name() for e in plotItem.listDataItems()]
    new_name = unique_data_item_name(name, existing)
    if new_name != name:
        dataItem.opts["name"] = new_name
    return dataItem



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

from datetime import datetime

import numpy
from taurus.external.qt import Qt
from taurus.qt.qtcore.configuration import BaseConfigurableClass
from taurus_pyqtgraph import DateAxisItem
from pyqtgraph import SignalProxy, InfiniteLine, TextItem, PlotDataItem


class DataInspectorLine(InfiniteLine):
    """
    DataInspectorLine provides a moveable vertical line item that shows labels
    containing the coordinates of the points of existing curves it touches.
    It provides a method to attach it to a PlotItem.

    """

    # TODO: modify anchor of labels so that they are plotted on the left if
    #       they do not fit in the view

    def __init__(
        self,
        date_format="%Y-%m-%d %H:%M:%S",
        x_format="0.4f",
        y_format="0.4f",
        trigger_point_size=10,
    ):
        """

        :param date_format: format string (as in strftime) for displaying dates
        :param x_format: format specifier for displaying the x values
        :param y_format: format specifier for displaying the y values
        :param trigger_point_size: width (in pixels) of the inspected area
                                   (points will be picked only if they are
                                   within this size)
        """
        super(DataInspectorLine, self).__init__(angle=90, movable=True)
        self._labels = []
        self._highlights = []
        self._plot_item = None

        self._date_format = "{:" + date_format + "}"
        self._x_format = "{:" + x_format + "}"
        self._y_format = "{:" + y_format + "}"
        self.trigger_point_size = trigger_point_size
        self._label_style = "background-color: #35393C;"
        self.sigPositionChanged.connect(self._inspect)

    def _inspect(self):
        """
        check if the line position matches (in widget coordinates) the position
        of a point of a given curve. If so, add a highlight point and a label
        in the corresponding viewbox.
        """
        # remove labels (text and highlight points)
        self._removeLabels()
        # group curves to be inspected, by their viewbox
        vb_curves = {}
        for c in self._plot_item.curves:
            if getattr(c, "xData", None) is not None:
                vb = c.getViewBox()
                if vb not in vb_curves:
                    vb_curves[vb] = []
                vb_curves[vb].append(c)

        # inspect the curves of each viewbox
        for vb, curves in vb_curves.items():
            self._inspect_curves_in_viewbox(curves, vb)

    def _inspect_curves_in_viewbox(self, curves, viewbox):

        # map position of the line to the viewbox coordinates
        xpos = viewbox.mapFromItemToView(self, self.pos()).x()
        # find out the screen pixel size in the viewbox coordinates
        x_px_size, _ = viewbox.viewPixelSize()

        points = []  # picked points, grouped by their viewbox
        # iterate over the curves
        for c in curves:
            # find the index of the closest point of this curve
            adiff = numpy.abs(c.xData - xpos)
            idx = numpy.argmin(adiff)
            # only add a label if the closest point is within tolerance
            tolerance = 0.5 * self.trigger_point_size * x_px_size
            if adiff[idx] < tolerance:
                points.append((c.xData[idx], c.yData[idx]))

        if curves:
            logMode = curves[0].opts["logMode"]
        else:
            logMode = None
        self._createLabels(points, viewbox, logMode)

    def _createLabels(self, points, viewbox, logMode):
        for x, y in points:
            # fill the text
            xtext = self._getXText(x)
            ytext = self._getYText(y)
            text_item = TextItem()
            text_item.setHtml(
                (
                    "<div style='{}'> "
                    + "<span><b>x=</b>{} "
                    + "<span><b>y=</b>{}</span> "
                    + "</div>"
                ).format(self._label_style, xtext, ytext)
            )
            # add text_item in the right position (take into account log mode)
            if logMode[0]:
                x = numpy.log10(x)
            if logMode[1]:
                y = numpy.log10(y)
            text_item.setPos(x, y)
            self._labels.append(text_item)
            viewbox.addItem(text_item, ignoreBounds=True)
        # Add "highlight" marker at each point
        highlight = PlotDataItem(
            numpy.array(points),
            pen=None,
            symbol="s",
            symbolBrush="35393C88",
            pxMode=True,
            symbolSize=self.trigger_point_size,
        )
        # set log mode
        highlight.setLogMode(*logMode)
        # hack to make the CurvesPropertiesTool ignore the highlight points
        highlight._UImodifiable = False
        # Add it to the vbox and keep a reference
        viewbox.addItem(highlight, ignoreBounds=True)
        self._highlights.append(highlight)

    def _getXText(self, x):
        """
        Helper method converting x value to time if necessary

        :param x: current x value
        :return: time or normal x value (depends of the x axis type)
        """
        x_axis = self._plot_item.getAxis("bottom")
        if isinstance(x_axis, DateAxisItem):
            return self._date_format.format(datetime.utcfromtimestamp(x))
        else:
            return self._x_format.format(x)

    def _getYText(self, y):
        return self._y_format.format(y)

    def _removeLabels(self):
        # remove existing texts labels and highlights
        for item in self._labels + self._highlights:
            item.getViewBox().removeItem(item)
        self._labels = []
        self._highlights = []

    def attachToPlotItem(self, plot_item):
        """
        Method to attach :class:`DataInspectorLine` to the plot

        :param plot: to attach
        """
        self._plot_item = plot_item
        self._plot_item.addItem(self, ignoreBounds=True)

    def dettach(self):
        """
        Method use to detach the class:`DataInspectorLine` from the plot
        """
        self._removeLabels()
        self._plot_item.removeItem(self)
        self._plot_item = None


class DataInspectorTool(Qt.QWidgetAction, BaseConfigurableClass):
    """
    This tool inserts an action in the menu of the :class:`pyqtgraph.PlotItem`
    to which it is attached. When activated, the data inspection mode is
    entered (a :class:`DataInspectorLine` is added and it follows the mouse,
    allowing the user to inspect the coordinates of existing curves).
    It is implemented as an Action, and provides a method to attach it to a
    PlotItem.
    """

    def __init__(self, parent=None):
        BaseConfigurableClass.__init__(self)
        Qt.QWidgetAction.__init__(self, parent)
        self._cb = Qt.QCheckBox()
        self._cb.setText("Data Inspector")
        self._cb.toggled.connect(self._onToggled)
        self.setDefaultWidget(self._cb)

        self.plot_item = None
        self.enable = False
        self.data_inspector = DataInspectorLine()

        self.registerConfigProperty(self.isChecked, self.setChecked, "checked")

    def attachToPlotItem(self, plot_item):
        """
        Use this method to add this tool to a plot

        :param plot_item: (PlotItem)
        :param y2: (Y2ViewBox) instance of the Y2Viewbox attached to plot_item
                   if the axis change controls are to be used
        """
        self.plot_item = plot_item
        menu = plot_item.getViewBox().menu
        menu.addAction(self)

    def _onToggled(self):

        if not self.enable:
            self.data_inspector.attachToPlotItem(self.plot_item)
            # Signal Proxy which connect the movement of the mouse with
            # the onMoved method in the data inspector object
            self.proxy = SignalProxy(
                self.plot_item.scene().sigMouseMoved,
                rateLimit=60,
                slot=self._followMouse,
            )
            self.enable = True
            # auto-close the menu so that the user can start inspecting
            self.plot_item.getViewBox().menu.close()

        else:
            self.proxy.disconnect()
            self.data_inspector.dettach()
            self.enable = False

    def _followMouse(self, evt):
        mouse_pos = evt[0]
        inspector_x = self.plot_item.vb.mapSceneToView(mouse_pos).x()
        self.data_inspector.setPos(inspector_x)


if __name__ == "__main__":
    from taurus.qt.qtgui.application import TaurusApplication
    import pyqtgraph as pg
    import sys

    app = TaurusApplication()

    w = pg.PlotWidget(title="[hint: enable inspector mode from context menu]")

    w.plot(y=numpy.arange(4), pen="r", symbol="o")
    w.plot(y=numpy.random.rand(4), pen="c")
    w.plot(
        x=numpy.linspace(0, 3, 40),
        y=1 + numpy.random.rand(40),
        pen="w",
        symbol="+",
    )

    t = DataInspectorTool(w)
    t.attachToPlotItem(w.getPlotItem())

    w.show()
    sys.exit(app.exec_())



import taurus_pyqtgraph as tpg
from .test_trend import show_and_wait  # noqa


def test_y2legend(qtbot):
    """
    Check that legend items are not duplicated or lost on changes of Y axis
    """
    w = tpg.TaurusPlot()
    qtbot.addWidget(w)

    vb1 = w.getViewBox()
    vb2 = w._y2
    plot_item = w.getPlotItem()
    legend = plot_item.legend
    assert w[:] == []
    assert vb1.addedItems == []
    assert vb2.addedItems == []
    assert legend.items == []

    # Add a curve to Y1
    w.setModel([(None, "eval:1*rand(11)", "foo")])
    c = w[0]
    assert w[:] == [c]
    assert vb1.addedItems == [c]
    assert vb2.addedItems == []
    assert len(legend.items) == 1
    assert legend.items[0][1].text == "foo"

    # move c to Y2
    tpg.set_y_axis_for_curve(
        y2=True, dataItem=c, plotItem=plot_item, y2Axis=w._y2
    )
    assert w[:] == [c]
    assert vb1.addedItems == []
    assert vb2.addedItems == [c]
    assert len(legend.items) == 1
    assert legend.items[0][1].text == "foo"

    # move c back to Y1
    tpg.set_y_axis_for_curve(
        y2=False, dataItem=c, plotItem=plot_item, y2Axis=w._y2
    )
    assert w[:] == [c]
    assert vb1.addedItems == [c]
    assert vb2.addedItems == []
    assert len(legend.items) == 1
    assert legend.items[0][1].text == "foo"



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""
curvesmodel Model and view for new CurveItem configuration

.. warning:: this is Work-in-progress. The API may change.
             Do not rely on current API of this module
"""
from __future__ import print_function
from future.utils import string_types
from builtins import bytes

__all__ = ["TaurusCurveItemTableModel", "TaurusItemConf", "TaurusItemConfDlg"]

import copy

from taurus.external.qt import Qt

import taurus
from taurus.core import TaurusElementType
from taurus.qt.qtcore.mimetypes import (
    TAURUS_MODEL_LIST_MIME_TYPE,
    TAURUS_ATTR_MIME_TYPE,
)
from taurus.qt.qtgui.util.ui import UILoadable
from taurus.qt.qtgui.panel import TaurusModelSelector

# columns:
NUMCOLS = 3
X, Y, TITLE = list(range(NUMCOLS))
SRC_ROLE = Qt.Qt.UserRole + 1


class Component(object):
    def __init__(self, src):
        self.display = ""
        self.icon = Qt.QIcon()
        self.ok = True
        self.processSrc(src)

    def processSrc(self, src):
        """
        processes the src and sets the values of display, icon and ok
        attributes
        """
        if src is None:
            self.display, self.icon, self.ok = "", Qt.QIcon(), True
            return
        src = str(src).strip()
        # empty
        if src == "":
            self.display, self.icon, self.ok = "", Qt.QIcon(), True
            return
        # for taurus attributes
        if taurus.isValidName(src, etypes=[TaurusElementType.Attribute]):
            self.display, self.icon, self.ok = (
                src,
                Qt.QIcon("logos:taurus.png"),
                True,
            )
            return

        # if not caught before, it is unsupported
        self.display, self.icon, self.ok = (
            src,
            Qt.QIcon.fromTheme("dialog-warning"),
            False,
        )


class TaurusItemConf(object):
    """An object to hold an item of the TaurusCurveItemTableModel"""

    def __init__(self, YModel=None, XModel=None, name=None):
        self.x = Component(XModel)
        self.y = Component(YModel)
        self.xModel = XModel
        self.yModel = YModel
        self.curveLabel = name

    def __repr__(self):
        ret = "TaurusItemConf(xModel='%s', yModel='%s')" % (
            self.xModel,
            self.yModel,
        )
        return ret


class TaurusCurveItemTableModel(Qt.QAbstractTableModel):
    """ A Qt data model for describing curves"""

    dataChanged = Qt.pyqtSignal("QModelIndex", "QModelIndex")

    def __init__(self, taurusItems=None):
        super(TaurusCurveItemTableModel, self).__init__()
        self.ncolumns = NUMCOLS
        self.taurusItems = list(taurusItems)

    def dumpData(self):
        return copy.copy(self.taurusItems)

    def rowCount(self, index=Qt.QModelIndex()):
        return len(self.taurusItems)

    def columnCount(self, index=Qt.QModelIndex()):
        return self.ncolumns

    def swapItems(self, index1, index2):
        """ swap the items described by index1 and index2 in the list"""
        r1, r2 = index1.row(), index2.row()
        items = self.taurusItems
        self.layoutAboutToBeChanged.emit()
        items[r1], items[r2] = items[r2], items[r1]
        self.dataChanged.emit(index1, index2)
        self.layoutChanged.emit()

    def data(self, index, role=Qt.Qt.DisplayRole):
        if not index.isValid() or not (0 <= index.row() < self.rowCount()):
            return None
        row = index.row()
        column = index.column()
        # Display Role
        if role == Qt.Qt.DisplayRole:
            if column == X:
                return str(self.taurusItems[row].x.display)
            elif column == Y:
                return str(self.taurusItems[row].y.display)
            elif column == TITLE:
                return str(self.taurusItems[row].curveLabel)
            else:
                return None
        elif role == Qt.Qt.DecorationRole:
            if column == X:
                return self.taurusItems[row].x.icon
            elif column == Y:
                return self.taurusItems[row].y.icon
            else:
                return None
        elif role == Qt.Qt.TextColorRole:
            if column == X:
                Qt.QColor(self.taurusItems[row].x.ok and "green" or "red")
            elif column == Y:
                Qt.QColor(self.taurusItems[row].y.ok and "green" or "red")
            else:
                return None
        elif role == SRC_ROLE:
            if column == X:
                return str(self.taurusItems[row].xModel)
            elif column == Y:
                return str(self.taurusItems[row].yModel)
            else:
                return None
        elif role == Qt.Qt.ToolTipRole:
            if column == X:
                return str(self.taurusItems[row].xModel)
            elif column == Y:
                return str(self.taurusItems[row].yModel)
            else:
                return None
        if role == Qt.Qt.EditRole:
            if column == X:
                return str(self.taurusItems[row].xModel)
            elif column == Y:
                return str(self.taurusItems[row].yModel)
            elif column == TITLE:
                return str(self.taurusItems[row].curveLabel)
            else:
                return None
        return None

    def headerData(self, section, orientation, role=Qt.Qt.DisplayRole):
        if role == Qt.Qt.TextAlignmentRole:
            if orientation == Qt.Qt.Horizontal:
                return int(Qt.Qt.AlignLeft | Qt.Qt.AlignVCenter)
            return int(Qt.Qt.AlignRight | Qt.Qt.AlignVCenter)
        if role != Qt.Qt.DisplayRole:
            return None
        # So this is DisplayRole...
        if orientation == Qt.Qt.Horizontal:
            if section == X:
                return "X source"
            elif section == Y:
                return "Y Source"
            elif section == TITLE:
                return "Title"
            return None
        else:
            return str(section + 1)

    def flags(self, index):
        # use this to set the editable flag when fix is selected
        if not index.isValid():
            return Qt.Qt.ItemIsEnabled
        column = index.column()
        if column in (X, Y):
            return Qt.Qt.ItemFlags(
                Qt.Qt.ItemIsEnabled
                | Qt.Qt.ItemIsEditable
                | Qt.Qt.ItemIsDragEnabled
                | Qt.Qt.ItemIsDropEnabled
                | Qt.Qt.ItemIsSelectable
            )
        elif column == TITLE:
            return Qt.Qt.ItemFlags(
                Qt.Qt.ItemIsEnabled
                | Qt.Qt.ItemIsEditable
                | Qt.Qt.ItemIsDragEnabled
            )
        return Qt.Qt.ItemFlags(
            Qt.Qt.ItemIsEnabled
            | Qt.Qt.ItemIsEditable
            | Qt.Qt.ItemIsDragEnabled
        )

    def setData(self, index, value=None, role=Qt.Qt.EditRole):
        if index.isValid() and (0 <= index.row() < self.rowCount()):
            row = index.row()
            curve = self.taurusItems[row]
            column = index.column()
            if column == X:
                curve.xModel = value
                curve.x.processSrc(value)
            elif column == Y:
                curve.yModel = value
                curve.y.processSrc(value)
            elif column == TITLE:
                curve.curveLabel = value
            self.dataChanged.emit(index, index)
            return True
        return False

    def insertRows(self, position=None, rows=1, parentindex=None):
        if position is None:
            position = self.rowCount()
        if parentindex is None:
            parentindex = Qt.QModelIndex()
        self.beginInsertRows(parentindex, position, position + rows - 1)
        slice = [TaurusItemConf() for i in range(rows)]
        self.taurusItems = (
            self.taurusItems[:position] + slice + self.taurusItems[position:]
        )
        self.endInsertRows()
        return True

    def removeRows(self, position, rows=1, parentindex=None):
        if parentindex is None:
            parentindex = Qt.QModelIndex()
        self.beginResetModel()
        self.beginRemoveRows(parentindex, position, position + rows - 1)
        self.taurusItems = (
            self.taurusItems[:position] + self.taurusItems[position + rows :]
        )
        self.endRemoveRows()
        self.endResetModel()
        return True

    def clearAll(self):
        self.removeRows(0, self.rowCount())

    def mimeTypes(self):
        result = list(Qt.QAbstractTableModel.mimeTypes(self))
        result += [TAURUS_ATTR_MIME_TYPE, "text/plain"]
        return result

    def dropMimeData(self, data, action, row, column, parent):
        if row == -1:
            if parent.isValid():
                row = parent.row()
            else:
                row = parent.rowCount()
        if column == -1:
            if parent.isValid():
                column = parent.column()
            else:
                column = parent.columnCount()
        if data.hasFormat(TAURUS_ATTR_MIME_TYPE):
            model = bytes(data.data(TAURUS_ATTR_MIME_TYPE)).decode("utf-8")
            self.setData(self.index(row, column), value=model)
            return True
        elif data.hasFormat(TAURUS_MODEL_LIST_MIME_TYPE):
            d = bytes(data.data(TAURUS_MODEL_LIST_MIME_TYPE))
            models = d.decode("utf-8").split()
            if len(models) == 1:
                self.setData(self.index(row, column), value=models[0])
                return True
            else:
                self.insertRows(row, len(models))
                for i, m in enumerate(models):
                    self.setData(self.index(row + i, column), value=m)
                return True
        elif data.hasText():
            self.setData(self.index(row, column), data.text())
            return True
        return False

    def mimeData(self, indexes):
        mimedata = Qt.QAbstractTableModel.mimeData(self, indexes)
        if len(indexes) == 1:
            #            mimedata.setData(TAURUS_ATTR_MIME_TYPE, data)
            data = self.data(indexes[0], role=SRC_ROLE)
            mimedata.setText(data)
        return mimedata
        # mimedata.setData()


@UILoadable(with_ui="ui")
class TaurusItemConfDlg(Qt.QWidget):
    """ A configuration dialog for creating new CurveItems.

    Provides a TaurusModelBrowser for Taurus models and an editable
    table for the sources and title of data
    """

    dataChanged = Qt.pyqtSignal("QModelIndex", "QModelIndex")
    applied = Qt.pyqtSignal()

    def __init__(self, parent=None, taurusItemsConf=None, showXcol=True):
        super(TaurusItemConfDlg, self).__init__(parent)
        self.loadUi()
        self._showXcol = showXcol

        if taurusItemsConf is None:
            taurusItemsConf = [
                TaurusItemConf(YModel=None, XModel=None, name=None)
            ]

        # @todo: The action for this button is not yet implemented
        self.ui.reloadBT.setEnabled(False)

        self.model = TaurusCurveItemTableModel(taurusItemsConf)

        self._toolbar = Qt.QToolBar("Selector toolbar")
        self.ui.horizontalLayout_2.addWidget(self._toolbar)
        self._toolbar.setIconSize(Qt.QSize(16, 16))
        self._toolbar.addAction(
            Qt.QIcon.fromTheme("list-add"), "Add row", self._onAddAction
        )
        self._removeAction = self._toolbar.addAction(
            Qt.QIcon.fromTheme("list-remove"),
            "Remove selected row",
            self._onRemoveThisAction,
        )
        self._removeAllAction = self._toolbar.addAction(
            Qt.QIcon.fromTheme("edit-clear"),
            "Remove all rows",
            self._onClearAll,
        )
        self._moveUpAction = self._toolbar.addAction(
            Qt.QIcon.fromTheme("go-up"),
            "Move up the row",
            self._onMoveUpAction,
        )
        self._moveDownAction = self._toolbar.addAction(
            Qt.QIcon.fromTheme("go-down"),
            "Move down the row",
            self._onMoveDownAction,
        )

        table = self.ui.curvesTable
        table.setModel(self.model)
        table.setColumnHidden(X, not self._showXcol)

        selectionmodel = table.selectionModel()
        selectionmodel.selectionChanged.connect(self._onSelectionChanged)

        # -------------------------------------------------------------------
        # I get "UnboundLocalError: local variable 'taurus' referenced before
        # assignment" if I don't import taurus again here
        # TODO: check if this workaround is really needed
        import taurus  # noqa

        # -------------------------------------------------------------------

        modelSelector = TaurusModelSelector(parent=self)
        self.ui.verticalLayout.addWidget(modelSelector)

        # Connections
        self.ui.applyBT.clicked.connect(self.onApply)
        self.ui.reloadBT.clicked.connect(self.onReload)
        self.ui.cancelBT.clicked.connect(self.close)
        self.ui.curvesTable.customContextMenuRequested.connect(
            self.onTableContextMenu
        )
        modelSelector.modelsAdded.connect(self.onModelsAdded)

    def onTableContextMenu(self, pos):
        index = self.ui.curvesTable.indexAt(pos)
        row = index.row()
        menu = Qt.QMenu(self.ui.curvesTable)
        if row >= 0:
            menu.addAction(
                Qt.QIcon.fromTheme("list-remove"),
                "Remove this curve",
                self._onRemoveThisAction,
            )
        menu.addAction(
            Qt.QIcon.fromTheme("edit-clear"), "Clear all", self.model.clearAll
        )
        menu.addAction(
            Qt.QIcon.fromTheme("list-add"),
            "Add new row",
            self.model.insertRows,
        )

        menu.exec_(Qt.QCursor.pos())

    def _onSelectionChanged(self):
        """ Modify the status of the actions"""
        selected = self.ui.curvesTable.selectedIndexes()
        rows = []
        for item in selected:
            if item.row() not in rows:
                rows.append(item.row())
        lrows = len(rows)
        row = self.ui.curvesTable.currentIndex().row()
        isLastElem = row == self.model.rowCount() - 1
        isFirstElem = row == 0
        self._removeAction.setEnabled(lrows == 1)
        self._moveUpAction.setEnabled(lrows == 1 and not isFirstElem)
        self._moveDownAction.setEnabled(lrows == 1 and not isLastElem)

    def _onAddAction(self):
        """ Add a new row"""
        self.model.insertRows()
        self._removeAllAction.setEnabled(True)

    def _onRemoveThisAction(self):
        """ Remove the selected row"""
        row = self.ui.curvesTable.currentIndex().row()
        self.model.removeRows(row)
        if self.model.rowCount() == 0:
            self._removeAllAction.setEnabled(False)

    def _onClearAll(self):
        """ Remove all rows"""
        self.model.clearAll()
        self._removeAction.setEnabled(False)
        self._moveUpAction.setEnabled(False)
        self._moveDownAction.setEnabled(False)
        self._removeAllAction.setEnabled(False)

    def _onMoveUpAction(self):
        """ Move up action swap the selected row with the previous one"""
        i1 = self.ui.curvesTable.currentIndex()
        i2 = self.ui.curvesTable.model().index(i1.row() - 1, 0)
        self.__commitAndCloseEditor(i1)
        self.model.swapItems(i1, i2)
        self.ui.curvesTable.setCurrentIndex(i2)

    def _onMoveDownAction(self):
        """ Move down action swap the selected row with the next one"""
        i1 = self.ui.curvesTable.currentIndex()
        i2 = self.ui.curvesTable.model().index(i1.row() + 1, 0)
        self.__commitAndCloseEditor(i1)
        self.model.swapItems(i1, i2)
        self.ui.curvesTable.setCurrentIndex(i2)

    def __commitAndCloseEditor(self, idx):
        """if an editor is open, commit the data and close it before moving

        :param idx: qmodel index
        """
        w = self.ui.curvesTable.indexWidget(idx)
        if w is not None:
            self.ui.curvesTable.commitData(w)
            self.ui.curvesTable.closePersistentEditor(idx)

    def onModelsAdded(self, models):
        nmodels = len(models)
        rowcount = self.model.rowCount()
        self.model.insertRows(rowcount, nmodels)
        for i, m in enumerate(models):
            if isinstance(m, string_types):
                modelx, modely = None, m
            else:
                modelx, modely = m

            if modelx is not None:
                self.model.setData(
                    self.model.index(rowcount + i, X), value=modelx
                )

            self.model.setData(self.model.index(rowcount + i, Y), value=modely)
            title = self.model.data(
                self.model.index(rowcount + i, Y)
            )  # the display data

            # print type(title), title
            self.model.setData(
                self.model.index(rowcount + i, TITLE), value=title
            )

    def getItemConfs(self):
        return self.model.dumpData()

    @staticmethod
    def showDlg(parent=None, taurusItemConf=None, showXCol=True):
        """
        Static method that launches a modal dialog containing a
        TaurusItemConfDlg.

        For the parameters, see :class:`TaurusItemConfDlg`

        :return: (list,bool) Returns a models,ok tuple
                 models is a list of models.
                 ok is True if the dialog was accepted (by clicking on the
                 "update" button) and False otherwise
        """
        dlg = Qt.QDialog(parent)
        dlg.setWindowTitle("Curves Selection")
        layout = Qt.QVBoxLayout()
        w = TaurusItemConfDlg(
            parent=parent, taurusItemsConf=taurusItemConf, showXcol=showXCol
        )
        layout.addWidget(w)
        dlg.setLayout(layout)
        w.applied.connect(dlg.accept)
        w.ui.cancelBT.clicked.connect(dlg.close)
        dlg.exec_()
        return w.getItemConfs(), (dlg.result() == dlg.Accepted)

    def onApply(self):
        self.applied.emit()

    def onReload(self):
        # TODO
        print("RELOAD!!! (todo)")


if __name__ == "__main__":
    from taurus.qt.qtgui.application import TaurusApplication

    import sys

    app = TaurusApplication(cmd_line_parser=None)

    TaurusItemConfDlg.showDlg()

    sys.exit(app.exec_())



import taurus_pyqtgraph as tpg
import pytest


class _DummyPlotItem:
    def __init__(self, names):
        self.ditems = [_DummyDataItem(n) for n in names]

    def listDataItems(self):
        return self.ditems[:]


class _DummyDataItem:
    def __init__(self, name):
        self.opts = {"name": name}

    def name(self):
        return self.opts["name"]


@pytest.mark.parametrize(
    "name,existing,expected",
    [
        ("a", list("qwerty"), "a"),
        ("a", list("asdfgh"), "a (2)"),
        ("a", ["a", "a (2)", "a (3)", "b"], "a (4)"),
    ],
)
def test_unique_data_item_name(name, existing, expected):
    assert tpg.unique_data_item_name(name, existing) == expected


@pytest.mark.parametrize(
    "name,existing,expected",
    [
        ("a", list("qwerty"), "a"),
        ("a", list("asdfgh"), "a (2)"),
        ("a", ["a", "a (2)", "a (3)", "b"], "a (4)"),
    ],
)
def test_ensure_unique_curve_name(name, existing, expected):
    plotItem = _DummyPlotItem(existing)
    dataItem = _DummyDataItem(name)
    out = tpg.ensure_unique_curve_name(dataItem, plotItem)
    assert out.name() == expected



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `taurus_pyqtgraph` package."""


from click.testing import CliRunner


def test_smoke():
    import taurus_pyqtgraph  # noqa


def test_command_line_interface():
    """Test the CLI."""
    from taurus_pyqtgraph import cli

    runner = CliRunner()
    result = runner.invoke(cli.tpg)
    assert result.exit_code == 0
    assert (
        '[DEPRECATED] use "taurus plot" or "taurus trend" instead'
        in result.output
    )
    help_result = runner.invoke(cli.tpg, ["--help"])
    assert help_result.exit_code == 0
    assert "Show this message and exit." in help_result.output



"""
Convenience functions, etc for tests
"""


def show_and_wait(qtbot, *widgets, timeout=60000, raising=False):
    """
    Helper that shows widgets and waits until they are closed (or timeout ms)
    """
    for w in widgets:
        w.show()

    def are_closed():
        for w in widgets:
            if w.isVisible():
                return False
        return True

    try:
        qtbot.wait_until(are_closed, timeout=timeout)
    except AssertionError:
        if raising:
            raise


def get_sub_config(cfg, item):
    assert item in cfg["__itemConfigurations__"]
    assert item in cfg["__orderedConfigNames__"]
    return cfg["__itemConfigurations__"][item]



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

__all__ = ["TaurusModelChooserTool", "TaurusImgModelChooserTool"]

from builtins import bytes
from future.utils import string_types

from taurus.external.qt import Qt
from taurus.core import TaurusElementType
from taurus.qt.qtcore.configuration import BaseConfigurableClass
from taurus.qt.qtgui.panel import TaurusModelChooser
from taurus_pyqtgraph.taurusimageitem import TaurusImageItem
from taurus_pyqtgraph.taurusplotdataitem import TaurusPlotDataItem
from taurus_pyqtgraph.curvesmodel import TaurusItemConf, TaurusItemConfDlg
from taurus_pyqtgraph.util import ensure_unique_curve_name
import taurus
from collections import OrderedDict
from taurus.qt.qtcore.mimetypes import (
    TAURUS_MODEL_LIST_MIME_TYPE,
    TAURUS_ATTR_MIME_TYPE,
)


class TaurusModelChooserTool(Qt.QAction):
    """
    This tool inserts an action in the menu of the :class:`pyqtgraph.PlotItem`
    to which it is attached to show choosing taurus models to be shown.
    It is implemented as an Action, and provides a method to attach it to a
    PlotItem.
    """

    def __init__(self, parent=None, itemClass=None):
        Qt.QAction.__init__(self, "Model selection", parent)
        self.triggered.connect(self._onTriggered)
        self.plot_item = None
        self.legend = None
        if itemClass is None:
            itemClass = TaurusPlotDataItem
        self.itemClass = itemClass

    def attachToPlotItem(self, plot_item, parentWidget=None):
        """
        Use this method to add this tool to a plot

        :param plot_item: (PlotItem)
        """
        self.plot_item = plot_item
        if self.plot_item.legend is not None:
            self.legend = self.plot_item.legend

        menu = self.plot_item.getViewBox().menu
        menu.addAction(self)
        self.setParent(parentWidget or menu)

    def _onTriggered(self):
        currentModelNames = []
        for item in self.plot_item.items:
            if isinstance(item, self.itemClass):
                currentModelNames.append(item.getFullModelName())
        names, ok = TaurusModelChooser.modelChooserDlg(
            selectables=[TaurusElementType.Attribute],
            listedModels=currentModelNames,
        )
        if ok:
            self.updateModels(names)

    def updateModels(self, names):
        """Accepts a list of model names and updates the data items of class
        `itemClass` (provided in the constructor) attached to the plot.
        It creates and removes items if needed, and enforces the z-order
        according to that given in the `models` list
        """
        # from names, construct an ordered dict with k=fullname, v=modelObj
        models = OrderedDict()
        for n in names:
            m = taurus.Attribute(n)
            models[m.getFullName()] = m

        # construct a dict and a list for current models and names
        currentModelItems = dict()
        currentModelNames = []
        for item in self.plot_item.items:
            if isinstance(item, self.itemClass):
                fullname = item.getFullModelName()
                currentModelNames.append(fullname)
                currentModelItems[fullname] = item

        # remove existing curves from plot (but not discarding the object)
        # so that they can be re-added later in the correct z-order
        for k, v in currentModelItems.items():
            # v.getViewBox().removeItem(v)  # TODO: maybe needed for Y2
            self.plot_item.removeItem(v)
            # -------------------------------------------------
            # Workaround for bug in pyqtgraph 0.10.0
            # (which is fixed in pyqtgraph's commit ee0ea5669)
            # TODO: remove this lines when pyqtgraph > 0.10.0 is released
            if self.legend is not None:
                self.legend.removeItem(v.name())
            # -------------------------------------------------

        # Add all curves (creating those that did not exist previously)
        # respecting the z-order
        for modelName, model in models.items():
            if modelName in currentModelNames:
                item = currentModelItems[modelName]
                self.plot_item.addItem(item)
                # item.getViewBox().addItem(item)  # TODO: maybe needed for Y2
            else:
                # TODO support labels
                item = self.itemClass(name=model.getSimpleName())
                item.setModel(modelName)
                self.plot_item.addItem(item)

        # self.plot_item.enableAutoRange()  # TODO: Why? remove?

    def setParent(self, parent):
        """Reimplement setParent to add an event filter"""
        Qt.QAction.setParent(self, parent)
        if parent is not None:
            parent.installEventFilter(self)

    def _dropMimeData(self, data):
        """Method to process the dropped MimeData"""
        ymodels = []
        if data.hasFormat(TAURUS_ATTR_MIME_TYPE):
            m = bytes(data.data(TAURUS_ATTR_MIME_TYPE)).decode("utf-8")
            ymodels.append(m)

        elif data.hasFormat(TAURUS_MODEL_LIST_MIME_TYPE):
            ymodels = (
                bytes(data.data(TAURUS_MODEL_LIST_MIME_TYPE))
                .decode("utf-8")
                .split()
            )
        elif data.hasText():
            ymodels.append(data.text())

        # Add models
        current = []
        for item in self.plot_item.items:
            if isinstance(item, self.itemClass):
                current.append(item.getFullModelName())
        self.updateModels(current + ymodels)
        return True

    def eventFilter(self, source, event):
        """
        Reimplementation of eventFilter to delegate parent's drag and drop
        events to TaurusModelChooserTool
        """
        if source is self.parent():
            if event.type() == Qt.QEvent.DragEnter:
                event.acceptProposedAction()
                return True

            if event.type() == Qt.QEvent.Drop:
                event.acceptProposedAction()
                return self._dropMimeData(event.mimeData())

        return self.parent().eventFilter(source, event)


class TaurusImgModelChooserTool(Qt.QAction):
    """
    This tool inserts an action in the menu of the :class:`pyqtgraph.PlotItem`
    to which it is attached for choosing a 2D taurus model to be shown.
    It is implemented as an Action, and provides a method to attach it to a
    PlotItem.
    """

    # TODO: merge this with TaurusModelChooserTool (or use a common base)

    def __init__(self, parent=None):
        Qt.QAction.__init__(self, parent)
        self._plot_item = None

    def attachToPlotItem(self, plot_item):
        """
        Use this method to add this tool to a plot

        :param plot_item: (PlotItem)
        """
        self._plot_item = plot_item
        view = plot_item.getViewBox()
        menu = view.menu
        model_chooser = Qt.QAction("Model selection", menu)
        model_chooser.triggered.connect(self._onTriggered)
        menu.addAction(model_chooser)

    def _onTriggered(self):

        imageItem = None

        for item in self._plot_item.items:
            if isinstance(item, TaurusImageItem):
                imageItem = item
                break

        if imageItem is None:
            imageItem = TaurusImageItem()
        modelName = imageItem.getFullModelName()
        if modelName is None:
            listedModels = []
        else:
            listedModels = [modelName]

        res, ok = TaurusModelChooser.modelChooserDlg(
            selectables=[TaurusElementType.Attribute],
            singleModel=True,
            listedModels=listedModels,
        )
        if ok:
            if res:
                model = res[0]
            else:
                model = None
            imageItem.setModel(model)


class TaurusXYModelChooserTool(Qt.QAction, BaseConfigurableClass):
    """
    (Work-in-Progress)
    This tool inserts an action in the menu of the :class:`pyqtgraph.PlotItem`
    to which it is attached for choosing X and Y 1D taurus models of the curves
    to be shown.
    It is implemented as an Action, and provides a method to attach it to a
    PlotItem.
    It only deals with the subgroup of plot data items of the type defined by
    `self.ItemClass` (which defaults to :class:`TaurusPlotDataItem`)
    """

    # TODO: This class is WIP.
    def __init__(self, parent=None, itemClass=None, showX=True):
        BaseConfigurableClass.__init__(self)
        Qt.QAction.__init__(self, "Model selection", parent)
        self.triggered.connect(self._onTriggered)
        self.plot_item = None
        self.legend = None
        self._curveColors = None
        if itemClass is None:
            itemClass = TaurusPlotDataItem
        self.itemClass = itemClass
        self._showX = showX

        self.registerConfigProperty(
            self._getCurveInfo, self._restoreCurvesFromInfo, "CurveInfo"
        )

    def setParent(self, parent):
        """Reimplement setParent to add an event filter"""
        Qt.QAction.setParent(self, parent)
        if parent is not None:
            parent.installEventFilter(self)

    def attachToPlotItem(
        self, plot_item, parentWidget=None, curve_colors=None
    ):
        """
        Use this method to add this tool to a plot

        :param plot_item: (PlotItem)

        .. warning:: this is Work-in-progress. The API may change.
             Do not rely on current signature of this method
        """
        # TODO: Check if we can simplify the signature (remove keyword args)
        self.plot_item = plot_item
        self._curveColors = curve_colors
        if self.plot_item.legend is not None:
            self.legend = self.plot_item.legend

        menu = self.plot_item.getViewBox().menu
        menu.addAction(self)
        self.setParent(parentWidget or menu)

    def _onTriggered(self):
        oldconfs = self._getTaurusPlotDataItemConfigs().values()
        newconfs, ok = TaurusItemConfDlg.showDlg(
            parent=self.parent(),
            taurusItemConf=oldconfs,
            showXCol=self._showX,
        )

        if ok:
            xy_names = [(c.xModel, c.yModel) for c in newconfs]
            self.updateModels(xy_names)
            # TODO: apply configurations too

    def _getTaurusPlotDataItemConfigs(self):
        """Get all the TaurusItemConf of the existing TaurusPlotDataItems
        Returns an ordered dict whose keys are (xfullname, yfullname)
        and whose values are the corresponding item config class
        """
        itemconfigs = OrderedDict()
        for curve in self.plot_item.listDataItems():
            if isinstance(curve, self.itemClass):
                xmodel, ymodel = curve.getFullModelNames()
                name = curve.name()
                # ugly hack for TaurusTrendset, which forces .name() ->None
                if name is None and hasattr(curve, "base_name"):
                    name = curve.base_name()
                c = TaurusItemConf(YModel=ymodel, XModel=xmodel, name=name)
                itemconfigs[(xmodel, ymodel)] = c
        return itemconfigs

    def _dropMimeData(self, data):
        """Method to process the dropped MimeData"""
        ymodels = []
        if data.hasFormat(TAURUS_ATTR_MIME_TYPE):
            m = bytes(data.data(TAURUS_ATTR_MIME_TYPE)).decode("utf-8")
            ymodels.append(m)

        elif data.hasFormat(TAURUS_MODEL_LIST_MIME_TYPE):
            ymodels = (
                bytes(data.data(TAURUS_MODEL_LIST_MIME_TYPE))
                .decode("utf-8")
                .split()
            )
        elif data.hasText():
            ymodels.append(data.text())

        xmodels = [None] * len(ymodels)
        self.addModels(list(zip(xmodels, ymodels)))
        return True

    def eventFilter(self, source, event):
        """
        Reimplementation of eventFilter to delegate parent's drag and drop
        events to TaurusXYModelChooserTool
        """
        if source is self.parent():
            if event.type() == Qt.QEvent.DragEnter:
                event.acceptProposedAction()
                return True

            if event.type() == Qt.QEvent.Drop:
                event.acceptProposedAction()
                return self._dropMimeData(event.mimeData())

        return self.parent().eventFilter(source, event)

    def getModelNames(self):
        """
        Get the x and y model names for the data items of type
        defined by `self.itemClass` present in the plot item to which
        this tool is attached
        """
        return [t[:2] for t in self._getCurveInfo()]

    def _getCurveInfo(self):
        """
        Get a list of (xmodelname, ymodelname, curvename) tuples for
        the data items of type defined by `self.itemClass` present in the
        PlotItem to which this tool is attached
        """
        cfgs = self._getTaurusPlotDataItemConfigs().values()
        return [(c.xModel, c.yModel, c.curveLabel) for c in cfgs]

    def _restoreCurvesFromInfo(self, info_tuples):
        self.updateModels(info_tuples)

    def addModels(self, xy_names):
        """Add new items with the given x and y model pairs. Those given model
        pairs that are already present will not be altered or duplicated (e.g.
        the z-order of the corresponding curve will not be modified for in case
        of adding duplicates)
        """
        current = self.getModelNames()
        self.updateModels(current + xy_names)

    def updateModels(self, xy_names):
        """
        Update the current plot item list with the given configuration items
        """
        mainViewBox = self.plot_item.getViewBox()
        # Remove existing taurus items from the plotItem and the viewboxes
        # (but keep the item object and a reference to their viewbox so that
        # they can be reused later on if needed).
        currentModelItems = OrderedDict()
        _currentItems = list(self.plot_item.listDataItems())
        for item in _currentItems:
            if isinstance(item, self.itemClass):
                xname, yname = item.getFullModelNames()
                viewbox = item.getViewBox()
                # store curve and current viewbox for later use
                currentModelItems[(xname, yname)] = item, viewbox
                # remove the curve
                self.plot_item.removeItem(item)
                if viewbox is not None and viewbox is not mainViewBox:
                    viewbox.removeItem(item)
                if self.legend is not None:
                    self.legend.removeItem(item.name())

        # Add only the curves defined in xy_names (reusing existing ones and
        # creating those that did not exist) in the desired z-order
        _already_added = []
        for xy_name in xy_names:
            # each member of xy_names can be yname or a (xname, yname[, cname])
            # tuple
            if isinstance(xy_name, string_types):
                xname, yname, cname = None, xy_name, None
            else:
                try:
                    xname, yname, cname = xy_name
                except ValueError:
                    xname, yname = xy_name
                    cname = None
            # make sure that fullnames are used
            try:
                if xname is not None:
                    xmodel = taurus.Attribute(xname)
                    xname = xmodel.getFullName()
                ymodel = taurus.Attribute(yname)
                yname = ymodel.getFullName()
            except Exception as e:
                from taurus import warning

                warning("Problem adding %r: %r", (xname, yname), e)
                continue
            # do not add it again if we already added it (avoid duplications)
            if (xname, yname) in _already_added:
                continue
            _already_added.append((xname, yname))
            # if the item already existed, re-use it
            if (xname, yname) in currentModelItems:
                item, viewbox = currentModelItems[(xname, yname)]
                self.plot_item.addItem(item)
                if viewbox is not mainViewBox:
                    # if the curve was originally associated to a viewbox
                    # other than the main one, we should move it there after
                    # re-adding it to the plot_item
                    mainViewBox.removeItem(item)
                    viewbox.addItem(item)
            # if it is a new curve, create it and add it to the main plot_item
            else:
                if cname is None:
                    cname = ymodel.getSimpleName()
                item = self.itemClass(
                    xModel=xname,
                    yModel=yname,
                    name=cname,
                    colors=self._curveColors,
                )
                item = ensure_unique_curve_name(item, self.plot_item)
                self.plot_item.addItem(item)


def _demo_ModelChooser():
    import sys
    import numpy
    import pyqtgraph as pg
    from taurus.qt.qtgui.tpg import TaurusModelChooserTool
    from taurus.qt.qtgui.application import TaurusApplication
    from taurus.qt.qtgui.tpg import TaurusPlotDataItem

    app = TaurusApplication()

    # a standard pyqtgraph plot_item
    w = pg.PlotWidget()

    # add legend to the plot, for that we have to give a name to plot items
    w.addLegend()

    # adding a regular data item (non-taurus)
    c1 = pg.PlotDataItem(name="st plot", pen="b", fillLevel=0, brush="c")
    c1.setData(numpy.arange(300) / 300.0)
    w.addItem(c1)

    # adding a taurus data item
    c2 = TaurusPlotDataItem(name="st2 plot", pen="r", symbol="o")
    c2.setModel("eval:rand(222)")

    w.addItem(c2)

    # attach to plot item
    tool = TaurusModelChooserTool(itemClass=TaurusPlotDataItem)
    tool.attachToPlotItem(w.getPlotItem())

    w.show()

    tool.trigger()

    sys.exit(app.exec_())


def _demo_ModelChooserImage():
    import sys
    from taurus.qt.qtgui.tpg import TaurusImgModelChooserTool, TaurusImageItem
    from taurus.qt.qtgui.application import TaurusApplication
    import pyqtgraph as pg

    app = TaurusApplication()

    w = pg.PlotWidget()

    img = TaurusImageItem()

    # Add taurus 2D image data
    img.setModel("eval:rand(256,256)")

    w.addItem(img)

    w.showAxis("left", show=False)
    w.showAxis("bottom", show=False)

    tool = TaurusImgModelChooserTool()
    tool.attachToPlotItem(w.getPlotItem())

    w.show()

    tool.trigger()
    sys.exit(app.exec_())


if __name__ == "__main__":
    _demo_ModelChooser()
    # _demo_ModelChooserImage()



#!/usr/bin/env python

##############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

from setuptools import setup, find_packages

description = "Taurus extension providing pyqtgraph-based widgets"

long_description = """taurus_pyqtgraph is an extension for the Taurus package.
It adds the taurus.qt.qtgui.tpg submodule which provides pyqtgraph-based
widgets."""

author = "Taurus Community"

maintainer = author

maintainer_email = "tauruslib-devel@lists.sourceforge.net"

url = "https://github.com/taurus-org/taurus_pyqtgraph"

download_url = url

platforms = ["Linux", "Windows"]

keywords = ["Taurus", "pyqtgraph", "plugin", "widgets"]

install_requires = ["pyqtgraph>=0.11", "click", "taurus>=4.5.2", "lxml", "ply"]

entry_points = {
    "taurus.qt.qtgui": ["tpg = taurus_pyqtgraph"],
    "taurus.cli.subcommands": ["tpg = taurus_pyqtgraph.cli:tpg"],
    "taurus.plot.alts": ["tpg = taurus_pyqtgraph:TaurusPlot"],
    "taurus.trend.alts": ["tpg = taurus_pyqtgraph:TaurusTrend"],
}

classifiers = [
    "Development Status :: 3 - Alpha",
    "Environment :: X11 Applications :: Qt",
    "Environment :: Win32 (MS Windows)",
    "Intended Audience :: Developers",
    "Intended Audience :: Science/Research",
    (
        "License :: OSI Approved :: "
        + "GNU Lesser General Public License v3 or later (LGPLv3+)"
    ),
    "Operating System :: Microsoft :: Windows",
    "Operating System :: POSIX",
    "Operating System :: POSIX :: Linux",
    "Operating System :: Unix",
    "Operating System :: OS Independent",
    "Natural Language :: English",
    "Programming Language :: Python :: 2",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
    "Topic :: Scientific/Engineering",
    "Topic :: Software Development :: Libraries",
    "Topic :: Software Development :: User Interfaces",
    "Topic :: Software Development :: Widget Sets",
]


setup(
    name="taurus_pyqtgraph",
    version="0.4.6",
    description=description,
    long_description=long_description,
    author=author,
    maintainer=maintainer,
    maintainer_email=maintainer_email,
    url=url,
    download_url=download_url,
    platforms=platforms,
    license="LGPLv3+",
    keywords=keywords,
    packages=find_packages(),
    classifiers=classifiers,
    include_package_data=True,
    entry_points=entry_points,
    test_suite="tests",
    python_requires=">=2.7",
    install_requires=install_requires,
)



import sys
import pytest
import numpy
import taurus_pyqtgraph as tpg
import pyqtgraph as pg
from taurus.external.qt import Qt

from .util import show_and_wait, get_sub_config  # noqa


def test_plot_model_setting(qtbot):

    w = tpg.TaurusPlot()
    qtbot.addWidget(w)

    assert len(w) == 0
    assert len(w._model_chooser_tool.getModelNames()) == 0

    models1 = [
        "eval:1*rand(22)",
        ("eval:linspace(-10,20,10)", "eval:2*rand(10)"),
    ]

    w.setModel(models1)
    c0 = w[0]
    c1 = w[1]
    assert len(w) == 2
    assert w._model_chooser_tool.getModelNames() == [
        c0.getFullModelNames(),
        c1.getFullModelNames(),
    ]

    # add a regular data item (non-taurus)
    c2 = pg.PlotDataItem(name="pg item", pen="b", fillLevel=0, brush="c")
    c2.setData(numpy.linspace(0, 20, 10))
    w.addItem(c2)
    assert w[:] == [c0, c1, c2]
    assert w._model_chooser_tool.getModelNames() == [
        c0.getFullModelNames(),
        c1.getFullModelNames(),
    ]

    # add a taurus data item
    c3 = tpg.TaurusPlotDataItem(name="taurus item", pen="r", symbol="o")
    c3.setModel('eval:Quantity(rand(16),"m")')
    w.addItem(c3)
    assert w[:] == [c0, c1, c2, c3]
    assert w._model_chooser_tool.getModelNames() == [
        c0.getFullModelNames(),
        c1.getFullModelNames(),
        c3.getFullModelNames(),
    ]

    # Add existing (c0) model again (it is ignored)
    w.addModels([models1[0]])
    assert w[:] == [c2, c0, c1, c3]  # there is reordering, non taurus first
    assert w._model_chooser_tool.getModelNames() == [
        c0.getFullModelNames(),
        c1.getFullModelNames(),
        c3.getFullModelNames(),
    ]

    # set 1 model (not adding!, the non-taurus curve is kept)
    w.setModel([models1[1]])
    assert w[:] == [c2, c1]  # c1 **is** still the same object!
    assert w._model_chooser_tool.getModelNames() == [c1.getFullModelNames()]

    # set empty model (not adding!, the non taurus curve is kept)
    w.setModel([])
    assert w[:] == [c2]
    assert w._model_chooser_tool.getModelNames() == []

    # remove non-taurus curve
    w.removeItem(c2)
    assert w[:] == []
    assert w._model_chooser_tool.getModelNames() == []

    # Uncomment for visual checks
    # show_and_wait(qtbot, w, timeout=3600000)


def test_plot_model_setting_with_y2(qtbot):

    w = tpg.TaurusPlot()
    qtbot.addWidget(w)
    vb1 = w.getViewBox()
    vb2 = w._y2

    assert w[:] == []
    assert w._model_chooser_tool.getModelNames() == []
    assert vb1.addedItems == []
    assert vb2.addedItems == []

    models1 = [
        "eval:1*rand(22)",
        ("eval:linspace(-10,20,10)", "eval:2*rand(10)"),
    ]

    w.setModel(models1)
    c0 = w[0]
    c1 = w[1]

    # move the first curve to Y2
    vb1.removeItem(c0)
    vb2.addItem(c0)
    assert vb1.addedItems == [c1]
    assert vb2.addedItems == [c0]

    assert w[:] == [c0, c1]
    assert w._model_chooser_tool.getModelNames() == [
        c0.getFullModelNames(),
        c1.getFullModelNames(),
    ]

    # add a regular data item (non-taurus) to y2
    c2 = pg.PlotDataItem(name="pg item", pen="b", fillLevel=0, brush="c")
    c2.setData(numpy.linspace(0, 20, 10))
    vb2.addItem(c2)

    assert vb1.addedItems == [c1]
    assert vb2.addedItems == [c0, c2]

    assert len(w) == 3
    assert w[:] == [c0, c1, c2]
    assert w._model_chooser_tool.getModelNames() == [
        c0.getFullModelNames(),
        c1.getFullModelNames(),
    ]

    # add a taurus data item to y2
    c3 = tpg.TaurusPlotDataItem(name="taurus item", pen="r", symbol="o")
    c3.setModel('eval:Quantity(rand(16),"m")')
    vb2.addItem(c3)
    assert vb1.addedItems == [c1]
    assert vb2.addedItems == [c0, c2, c3]
    assert w[:] == [c0, c1, c2, c3]
    assert w._model_chooser_tool.getModelNames() == [
        c0.getFullModelNames(),
        c1.getFullModelNames(),
        c3.getFullModelNames(),
    ]

    # Add existing (c0) model again
    w.addModels([models1[0]])
    assert vb1.addedItems == [c1]
    assert vb2.addedItems == [c2, c0, c3]
    assert w[:] == [c2, c0, c1, c3]  # there is reordering, non taurus first
    assert w._model_chooser_tool.getModelNames() == [
        c0.getFullModelNames(),
        c1.getFullModelNames(),
        c3.getFullModelNames(),
    ]

    # set (not adding!) 1 model which is already on y2
    # only non-taurus curve and the set curve remain. Both stay in y2
    w.setModel([models1[0]])
    assert vb1.addedItems == []
    assert vb2.addedItems == [c2, c0]
    assert w[:] == [c2, c0]  # c0 **is** still the same object!
    assert w._model_chooser_tool.getModelNames() == [c0.getFullModelNames()]

    # set empty model (not adding!, only the non-taurus curve remains)
    w.setModel([])
    assert vb1.addedItems == []
    assert vb2.addedItems == [c2]
    assert w[:] == [c2]
    assert w._model_chooser_tool.getModelNames() == []

    # remove non-taurus curve
    c2.getViewBox().removeItem(c2)
    w.getPlotItem().removeItem(c2)
    assert vb1.addedItems == []
    assert vb2.addedItems == []
    assert w[:] == []
    assert w._model_chooser_tool.getModelNames() == []

    # Uncomment for visual checks
    # show_and_wait(qtbot, w, timeout=3600000)


def test_multiple_setModel(qtbot):
    """
    Check that repeated calls to setModel do not duplicate the items
    in the plot
    """
    w = tpg.TaurusPlot()
    qtbot.addWidget(w)
    for i in range(5):
        w.setModel(["eval:rand(2)"])
        assert w.getPlotItem().listDataItems() == [
            w[0]
        ], "Found duplicates after {} calls to setModel".format(i + 1)


def test_xymodelchooser_config(qtbot):

    w1 = tpg.TaurusPlot()
    qtbot.addWidget(w1)

    models1 = [
        "eval:1*rand(22)",
        ("eval:linspace(-10,20,10)", "eval:2*rand(10)"),
    ]

    w1.setModel(models1)

    # test createConfig
    cfg = w1.createConfig()
    xymccfg1 = get_sub_config(cfg, "XYmodelchooser")
    modelscfg1 = get_sub_config(xymccfg1, "CurveInfo")
    assert modelscfg1[0] == (
        None,
        "eval://localhost/@DefaultEvaluator/1*rand(22)",
        "1*rand(22)",
    )
    assert modelscfg1[1] == (
        "eval://localhost/@DefaultEvaluator/linspace(-10,20,10)",
        "eval://localhost/@DefaultEvaluator/2*rand(10)",
        "2*rand(10)",
    )

    # test applyConfig
    w2 = tpg.TaurusPlot()
    qtbot.addWidget(w2)
    assert len(w2) == 0
    assert len(w2._model_chooser_tool.getModelNames()) == 0

    # add a model to w2
    w2.setModel("eval:9*rand(11)")
    assert len(w2) == 1
    assert len(w2._model_chooser_tool.getModelNames()) == 1

    # apply config (the previously added tauruscurve of w2 should be removed)
    w2.applyConfig(cfg)
    assert len(w2) == 2
    assert len(w2._model_chooser_tool.getModelNames()) == 2
    assert w2._model_chooser_tool._getCurveInfo() == modelscfg1

    # Uncomment for visual checks
    # show_and_wait(qtbot, w1, w2, timeout=3600000)


def test_y2_config(qtbot):

    # create a plot with 3 curves
    w1 = tpg.TaurusPlot()
    qtbot.addWidget(w1)

    models1 = [
        "eval:1*rand(11)",
        "eval:2*rand(22)",
        ("eval:linspace(-10,20,10)", "eval:2*rand(10)"),
    ]

    w1.setModel(models1)

    w1_vb1 = w1.getViewBox()
    w1_vb2 = w1._y2

    # check that the 3 curves are on Y1
    assert [c.getViewBox() for c in w1] == [w1_vb1] * 3
    assert len(w1_vb1.addedItems) == 3
    assert len(w1_vb2.addedItems) == 0

    # move the second curve to Y2
    c = w1[1]
    w1_vb1.removeItem(c)
    w1_vb2.addItem(c)

    # check that the move was ok and there are no duplicates
    assert [c.getViewBox() for c in w1] == [w1_vb1, w1_vb2, w1_vb1]
    assert w1_vb2._getCurvesNames() == [w1[1].getFullModelNames()]
    assert len(w1_vb1.addedItems) == 2
    assert len(w1_vb2.addedItems) == 1

    # test createConfig
    plot_cfg = w1.createConfig()
    y2_cfg = get_sub_config(plot_cfg, "Y2Axis")
    curvescfg1 = get_sub_config(y2_cfg, "Y2Curves")
    assert curvescfg1 == [w1[1].getFullModelNames()]

    # # Debugging
    # from pprint import pprint
    # pprint(plot_cfg)
    # pprint(curvescfg1)

    # create a second, empty plot
    w2 = tpg.TaurusPlot()
    qtbot.addWidget(w2)
    assert len(w2) == 0

    # check applyConfig on the new plot
    w2.applyConfig(plot_cfg)
    assert len(w2) == 3

    w2_vb1 = w2.getViewBox()
    w2_vb2 = w2._y2
    w1_all_names = [c.getFullModelNames() for c in w1]
    w2_all_names = [c.getFullModelNames() for c in w2]

    assert w1_all_names == w2_all_names
    assert len(w2_all_names) == 3
    assert len(w2_vb1.addedItems) == 2
    assert len(w2_vb2.addedItems) == 1
    assert w2_vb2._getCurvesNames() == w1_vb2._getCurvesNames()
    assert w2_vb2._getCurvesNames() == [w1[1].getFullModelNames()]

    # Uncomment for visual checks
    # show_and_wait(qtbot, w1, w2, timeout=3600000)


def test_curveproperties(qtbot):

    w = tpg.TaurusPlot()
    qtbot.addWidget(w)

    # add a regular data item (non-taurus) to y1
    c0 = pg.PlotDataItem(name="pg item1", pen="b", fillLevel=0, brush="c")
    c0.setData(numpy.linspace(0, 20, 10))
    w.addItem(c0)

    # add a regular data item (non-taurus) to y2
    c1 = pg.PlotDataItem(name="pg item2", pen="y", symbol="d", symbolBrush="r")
    c1.setData(20 - numpy.linspace(0, 20, 10))
    w._y2.addItem(c1)

    # add a taurus data item to y1
    c2 = tpg.TaurusPlotDataItem(name="taurus item1", pen="r", symbol="o")
    c2.setModel('eval:Quantity(rand(16),"m")')
    w.addItem(c2)

    # add a taurus data item to y2
    c3 = tpg.TaurusPlotDataItem(name="taurus item2", pen="y", symbol="s")
    c3.setModel('eval:Quantity(rand(20),"km")')
    w._y2.addItem(c3)

    # Add 2 more items using setModel
    models1 = [
        "eval:1*rand(22)",
        ("eval:linspace(-10,20,10)", "eval:2*rand(10)"),
    ]
    w.addModels(models1)
    c4 = w[4]
    c5 = w[5]

    c4.setPen("g")

    c5.setPen(None)
    c5.setSymbol("t")
    c5.setSymbolSize(7)

    # move c5 to y2
    w._y2.addItem(c5)

    assert len(w) == 6
    assert w[:] == [c0, c1, c2, c3, c4, c5]
    assert len(w._cprop_tool.getModifiableItems()) == 6
    for c in [c0, c1, c2, c3, c4, c5]:
        assert c.name() in w._cprop_tool.getModifiableItems().keys()
        assert c in w._cprop_tool.getModifiableItems().values()
    assert w.getViewBox().addedItems == [c0, c2, c4]
    assert w._y2.addedItems == [c1, c3, c5]

    prop = w._cprop_tool._getCurveAppearanceProperties()

    # check lColor
    assert pg.mkColor(prop[c0.name()].lColor) == pg.mkColor("b")
    assert pg.mkColor(prop[c1.name()].lColor) == pg.mkColor("y")
    assert pg.mkColor(prop[c2.name()].lColor) == pg.mkColor("r")
    assert pg.mkColor(prop[c3.name()].lColor) == pg.mkColor("y")
    assert pg.mkColor(prop[c4.name()].lColor) == pg.mkColor("g")
    # assert pg.mkColor(prop[c5.name()].lColor) == pg.mkColor('b')

    # check lStyle
    for c in c0, c1, c2, c3, c4:
        assert prop[c.name()].lStyle == Qt.Qt.SolidLine
    assert prop[c5.name()].lStyle == Qt.Qt.NoPen

    # check y2
    for c in c0, c2, c4:
        assert prop[c.name()].y2 is False
    for c in c1, c3, c5:
        assert prop[c.name()].y2 is True

    # Uncomment for visual checks
    # show_and_wait(qtbot, w, timeout=3600000)


@pytest.mark.xfail(
    sys.version_info[:2] == (3, 6),
    reason="flaky behaviour observed in CI for py 3.6 in this test",
)
def test_curveproperties_config(qtbot):
    w1 = tpg.TaurusPlot()
    qtbot.addWidget(w1)

    # add a regular data item (non-taurus) to y1
    c0 = pg.PlotDataItem(name="pg item1", pen="m", fillLevel=0, brush="c")
    c0.setData(numpy.linspace(0, 20, 10) / 20.0)
    w1.addItem(c0)

    # add a regular data item (non-taurus) to y2
    c1 = pg.PlotDataItem(name="pg item2", pen="y", symbol="d", symbolBrush="r")
    c1.setData(1 - numpy.linspace(0, 20, 10) / 20.0)
    w1._y2.addItem(c1)

    # add a taurus data item to y1
    c2 = tpg.TaurusPlotDataItem(name="taurus item1", pen="r", symbol="o")
    c2.setModel('eval:Quantity(rand(16),"m")')
    w1.addItem(c2)

    # add a taurus data item to y2
    c3 = tpg.TaurusPlotDataItem(name="taurus item2", pen="y", symbol="s")
    c3.setModel('eval:Quantity(rand(20),"km")')
    w1._y2.addItem(c3)

    # Add 2 more items using setModel
    models1 = [
        "eval:1*rand(22)",
        ("eval:linspace(-10,20,10)", "eval:2*rand(10)"),
    ]
    w1.addModels(models1)
    c4 = w1[4]
    c5 = w1[5]

    c4.setPen("g")

    c5.setPen(None)
    c5.setSymbol("t")
    c5.setSymbolSize(7)
    c5.setSymbolBrush("r")

    # move c5 to y2
    w1._y2.addItem(c5)

    assert len(w1) == 6
    assert w1[:] == [c0, c1, c2, c3, c4, c5]
    w1_mod_items = w1._cprop_tool.getModifiableItems()
    for c in [c0, c1, c2, c3, c4, c5]:
        assert c0 in w1_mod_items.values()
    assert w1.getViewBox().addedItems == [c0, c2, c4]
    assert w1._y2.addedItems == [c1, c3, c5]

    # test createConfig
    cfg = w1.createConfig()
    propcfg = get_sub_config(cfg, "CurvePropertiesTool")
    _ = get_sub_config(propcfg, "CurveProperties")

    # # Debugging
    # from pprint import pprint
    # # pprint(cfg)
    # print("-" * 80)
    # pprint(curvescfg)

    # test applyConfig
    w2 = tpg.TaurusPlot()
    qtbot.addWidget(w2)
    # assert len(w2._model_chooser_tool.getModelNames()) == 0
    assert len(w2._cprop_tool._getCurveAppearanceProperties()) == 0

    w2.applyConfig(cfg)
    # assert len(w2._model_chooser_tool.getModelNames()) == 2
    # assert w2._model_chooser_tool.getModelNames() == modelscfg1

    w1_props = w1._cprop_tool._getCurveAppearanceProperties()
    w2_props = w2._cprop_tool._getCurveAppearanceProperties()

    assert len(w1_props) == 6
    assert len(w2_props) == 4
    for k, p_aft in w2_props.items():
        assert k in w1_props
        p_ini = w1_props[k]
        conflicts = p_ini.conflictsWith(p_aft, strict=True)
        msg = "Mismatch in saved/restored curve properties for '{}':".format(k)
        msg += "\n\t Saved:    {}".format(p_ini)
        msg += "\n\t Restored: {}".format(p_aft)
        assert conflicts == [], msg

    # test applyConfig
    w3 = tpg.TaurusPlot()
    qtbot.addWidget(w3)

    # Manually add regular data items matching the names used in w1
    # but do not match the properties, which should be updated by applyConfig
    c0_w3 = pg.PlotDataItem(
        name="pg item1", y=numpy.linspace(0, 20, 10) / 20.0
    )
    w3.addItem(c0_w3)
    c1_w3 = pg.PlotDataItem(name="pg item2", y=numpy.zeros(15))
    w3._y2.addItem(c1_w3)

    assert len(w3._cprop_tool._getCurveAppearanceProperties()) == 2

    w3.applyConfig(cfg)

    w3_props = w3._cprop_tool._getCurveAppearanceProperties()

    assert len(w3_props) == 6
    for k, p_aft in w3_props.items():
        assert k in w1_props
        p_ini = w1_props[k]
        conflicts = p_ini.conflictsWith(p_aft, strict=True)
        msg = "Mismatch in saved/restored curve properties for '{}':".format(k)
        msg += "\n\t Saved:    {}".format(p_ini)
        msg += "\n\t Restored: {}".format(p_aft)
        assert conflicts == [], msg

    # Uncomment for visual checks
    # show_and_wait(qtbot, w1, w2, w3, timeout=3600000)


def test_curveproperties_configfile(qtbot, tmp_path):
    w1 = tpg.TaurusPlot()
    qtbot.addWidget(w1)
    w1.setBackground("y")

    # add a regular data item (non-taurus) to y1
    c0 = pg.PlotDataItem(name="pg item1", pen="m", fillLevel=0, brush="c")
    c0.setData(numpy.linspace(0, 20, 10) / 20.0)
    w1.addItem(c0)

    # add a regular data item (non-taurus) to y2
    c1 = pg.PlotDataItem(name="pg item2", pen="y", symbol="d", symbolBrush="r")
    c1.setData(1 - numpy.linspace(0, 20, 10) / 20.0)
    w1._y2.addItem(c1)

    # add a taurus data item to y1
    c2 = tpg.TaurusPlotDataItem(name="taurus item1", pen="r", symbol="o")
    c2.setModel('eval:Quantity(rand(16),"m")')
    w1.addItem(c2)

    # add a taurus data item to y2
    c3 = tpg.TaurusPlotDataItem(name="taurus item2", pen="y", symbol="s")
    c3.setModel('eval:Quantity(rand(20),"km")')
    w1._y2.addItem(c3)

    # Add 2 more items using setModel
    models1 = [
        "eval:1*rand(22)",
        ("eval:linspace(-10,20,10)", "eval:2*rand(10)"),
    ]
    w1.addModels(models1)
    c4 = w1[4]
    c5 = w1[5]

    c4.setPen("g")

    c5.setPen(None)
    c5.setSymbol("t")
    c5.setSymbolSize(7)
    c5.setSymbolBrush("r")

    # move c5 to y2
    w1._y2.addItem(c5)

    assert len(w1) == 6
    assert w1[:] == [c0, c1, c2, c3, c4, c5]
    w1_mod_items = w1._cprop_tool.getModifiableItems()
    for c in [c0, c1, c2, c3, c4, c5]:
        assert c0 in w1_mod_items.values()
    assert w1.getViewBox().addedItems == [c0, c2, c4]
    assert w1._y2.addedItems == [c1, c3, c5]

    # test saveConfigFile
    f = tmp_path / "plot.pck"
    with open(str(f), "wb") as ofile:
        w1.saveConfigFile(ofile=ofile)
    assert f.exists()
    assert len(list(tmp_path.iterdir())) == 1

    # test loadConfigFile
    w2 = tpg.TaurusPlot()
    qtbot.addWidget(w2)
    with open(str(f), "rb") as ifile:
        w2.loadConfigFile(ifile=ifile)

    w1_props = w1._cprop_tool._getCurveAppearanceProperties()
    w2_props = w2._cprop_tool._getCurveAppearanceProperties()

    assert len(w1_props) == 6
    assert len(w2_props) == 4
    for k, p_aft in w2_props.items():
        assert k in w1_props
        p_ini = w1_props[k]
        conflicts = p_ini.conflictsWith(p_aft, strict=True)
        msg = "Mismatch in saved/restored curve properties for '{}':".format(k)
        msg += "\n\t Saved:    {}".format(p_ini)
        msg += "\n\t Restored: {}".format(p_aft)
        assert conflicts == [], msg

    # Uncomment for visual checks
    # show_and_wait(qtbot, w1, w2, timeout=3600000)


# def test_save_config_action(qtbot, tmp_path):
#     w1 = tpg.TaurusPlot()
#     qtbot.addWidget(w1)
#
#     w2 = tpg.TaurusPlot()
#     qtbot.addWidget(w2)
#
#     menu = w1.getPlotItem().getViewBox().menu
#     save_action = None
#     for a in menu.actions():
#         # print(a.text())
#         if a.text() == "Save configuration":
#             save_action = a
#             break
#     assert save_action is not None
#     save_action.trigger()
#     # TODO: handle the modal dialog
#     menu = w2.getPlotItem().getViewBox().menu
#     load_action = None
#     for a in menu.actions():
#         # print(a.text())
#         if a.text() == "Retrieve saved configuration":
#             load_action = a
#             break
#     assert load_action is not None
#     load_action.trigger()
#     # TODO: handle the modal dialog


def test_duplicate_curve_titles(qtbot):
    """
    Check that curves with same title are changed to have unique titles
    """
    w = tpg.TaurusPlot()
    qtbot.addWidget(w)
    w.setModel(
        [
            "eval:@foo/a=0+rand(3);a",
            "eval:@bar/a=1+rand(3);a",
            (None, "eval:2+rand(3)", "a"),
            "eval:3+rand(3)",
        ]
    )

    assert len(w[:]) == 4
    mod_items = w._cprop_tool.getModifiableItems()
    assert len(mod_items) == 4, "{}".format(mod_items)
    assert list(sorted(mod_items.keys())) == [
        "3+rand(3)",
        "a",
        "a (2)",
        "a (3)",
    ]
    # show_and_wait(qtbot, w)



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################
__all__ = ["TaurusImageItem"]

import sys
from taurus.qt.qtgui.application import TaurusApplication
from taurus.qt.qtgui.base import TaurusBaseComponent
from pyqtgraph import ImageItem


class TaurusImageItem(ImageItem, TaurusBaseComponent):
    """
    Displays 2D and 3D image data
    """

    # TODO: clear image if .setModel(None)
    def __init__(self, *args, **kwargs):
        ImageItem.__init__(self, *args, **kwargs)
        TaurusBaseComponent.__init__(self, "TaurusImageItem")

    def handleEvent(self, evt_src, evt_type, evt_val):
        if evt_val is None or getattr(evt_val, "rvalue", None) is None:
            self.debug("Ignoring empty value event from %s" % repr(evt_src))
            return
        try:
            data = evt_val.rvalue
            self.setImage(data)
        except Exception as e:
            self.warning("Exception in handleEvent: %s", e)


if __name__ == "__main__":
    import pyqtgraph as pg

    app = TaurusApplication()

    plot_widget = pg.PlotWidget()
    plot_item = plot_widget.getPlotItem()

    image_item = TaurusImageItem()

    # Add taurus 2D image data
    image_item.setModel("eval:randint(0,256,(16,16))")

    # add TarusImageItem to a PlotItem
    plot_item.addItem(image_item)

    # show or hide axis from the plot
    plot_item.showAxis("left", show=True)
    plot_item.showAxis("bottom", show=True)

    plot_widget.show()

    sys.exit(app.exec_())
